from unittest import TestCase
import odmlib.odm_1_3_2.model as ODM
import odmlib.odm_parser as ODM_PARSER
import xml.etree.ElementTree as ET
import os
import datetime


class TestReferenceData(TestCase):
    def setUp(self) -> None:
        self.odm_test_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data','test_referece_data_01.xml')

    def test_reference_data_to_xml(self):
        rd = []
        rd.append(ODM.ReferenceData(StudyOID="P2006-101", MetaDataVersionOID="101.01"))
        rd[0].ItemGroupData.append(ODM.ItemGroupData(ItemGroupOID="IG.REFSAMP", ItemGroupRepeatKey="1", TransactionType="Insert"))
        rd[0].ItemGroupData[0].ItemData.append(ODM.ItemData(ItemOID="IT.REF1", Value="1"))
        rd[0].ItemGroupData[0].ItemData.append(ODM.ItemData(ItemOID="IT.REF2", Value="Sample reference data line 1"))
        rd[0].ItemGroupData.append(ODM.ItemGroupData(ItemGroupOID="IG.REFSAMP", ItemGroupRepeatKey="2", TransactionType="Insert"))
        rd[0].ItemGroupData[1].ItemData.append(ODM.ItemData(ItemOID="IT.REF1", Value="2"))
        rd[0].ItemGroupData[1].ItemData.append(ODM.ItemData(ItemOID="IT.REF2", Value="Sample reference data line 2"))
        root = self.create_odm_document(rd)
        odm_xml = root.to_xml()
        self.assertEqual(odm_xml.attrib["FileOID"], "ODM.TEST.RD.001")
        self.assertListEqual(["ReferenceData"], [e.tag for e in odm_xml.getchildren()])

    def test_clinical_data_from_xml(self):
        parser = ODM_PARSER.ODMParser(self.odm_test_file)
        parser.parse()
        rd = parser.ReferenceData()
        igd_list = parser.ItemGroupData(parent=rd[0])
        item_data = parser.ItemData(parent=igd_list[1]["elem"])
        self.assertEqual(item_data[0]["Value"], "2")

    def test_audit_records(self):
        ar = ODM.AuditRecord(EditPoint="Monitoring", UsedImputationMethod="Yes", ID="2")
        ar.UserRef = ODM.UserRef(UserOID="USER.001")
        ar.LocationRef = ODM.LocationRef(LocationOID="SITE.102")
        ar.DateTimeStamp = ODM.DateTimeStamp(_content="2020-12-01T10:28:16")
        ar.ReasonForChange = ODM.ReasonForChange(_content="Missing data")
        ar.SourceID = ODM.SourceID(_content="RECID14327")
        ars = ODM.AuditRecords()
        ars.AuditRecord.append(ar)
        rd = []
        rd.append(ODM.ReferenceData(StudyOID="P2006-101", MetaDataVersionOID="101.01"))
        rd[0].AuditRecords.append(ars)
        self.assertEqual(rd[0].AuditRecords[0].AuditRecord[0].ID, "2")
        self.assertEqual(rd[0].AuditRecords[0].AuditRecord[0].LocationRef.LocationOID, "SITE.102")

    def test_signatures(self):
        sig = ODM.Signature(ID="SIG.001.USER.002")
        sig.UserRef = ODM.UserRef(UserOID="USER.001")
        sig.LocationRef = ODM.LocationRef(LocationOID="SITE.101")
        sig.SignatureRef = ODM.SignatureRef(SignatureOID="SIG.002")
        sig.DateTimeStamp = ODM.DateTimeStamp(_content="2020-12-01T10:28:16")
        sigs = ODM.Signatures()
        sigs.Signature.append(sig)
        rd = []
        rd.append(ODM.ReferenceData(StudyOID="P2006-101", MetaDataVersionOID="101.01"))
        rd[0].Signatures.append(sigs)
        self.assertEqual(rd[0].Signatures[0].Signature[0].ID, "SIG.001.USER.002")
        self.assertEqual(rd[0].Signatures[0].Signature[0].SignatureRef.SignatureOID, "SIG.002")

    def test_annotations(self):
        annotation = ODM.Annotation(SeqNum="1", TransactionType="Insert", ID="ANN999")
        annotation.Comment = ODM.Comment(SponsorOrSite="Site", _content="Transferred from EHR")
        annotation.Flag.append(ODM.Flag())
        annotation.Flag[0].FlagValue = ODM.FlagValue(CodeListOID="CL.FLAGVALUE", _content="eSource")
        annotation.Flag[0].FlagType = ODM.FlagType(CodeListOID="CL.FLAGTYPE", _content="eDT")
        anns = ODM.Annotations()
        anns.Annotation.append(annotation)
        rd = []
        rd.append(ODM.ReferenceData(StudyOID="P2006-101", MetaDataVersionOID="101.01"))
        rd[0].Annotations.append(anns)
        self.assertEqual(rd[0].Annotations[0].Annotation[0].ID, "ANN999")
        self.assertEqual(rd[0].Annotations[0].Annotation[0].Flag[0].FlagValue.CodeListOID, "CL.FLAGVALUE")

    def create_odm_document(self, rd):
        """
        assemble the ODM document, add the ItemGroupDef, and write it to a file
        :param igd: ItemGroupDef object
        """
        root = self.create_root()
        for ref_data in rd:
            root.ReferenceData.append(ref_data)
        root.write_xml(self.odm_test_file)
        return root

    def create_root(self):
        """
        create the ODM root element object with test data
        :return: ODM root element object
        """
        root = {"FileOID": "ODM.TEST.RD.001", "Granularity": "Metadata", "AsOfDateTime": self.set_datetime(),
                "CreationDateTime": self.set_datetime(), "ODMVersion": "1.3.2", "Originator": "ODMLIB", "SourceSystem": "ODMLIB",
                "SourceSystemVersion": "0.1", "FileType": "Snapshot"}
        root = ODM.ODM(**root)
        return root

    def set_datetime(self):
        """return the current datetime in ISO 8601 format"""
        return datetime.datetime.utcnow().replace(tzinfo=datetime.timezone.utc).isoformat()



from unittest import TestCase
import odmlib.odm_1_3_2.model as ODM


class TestStudy(TestCase):
    def setUp(self) -> None:
        self.study = ODM.Study(OID="ST.001.Test")

    def test_required_attributes(self):
        self.assertEqual(self.study.OID, "ST.001.Test")

    def test_full_study(self):
        study_name = ODM.StudyName(_content="The ODM study name")
        study_desc = ODM.StudyDescription(_content="The description of the ODM study")
        protocol_name = ODM.ProtocolName(_content="The ODM protocol name")
        self.study.GlobalVariables = ODM.GlobalVariables()
        self.study.GlobalVariables.StudyName = study_name
        self.study.GlobalVariables.StudyDescription = study_desc
        self.study.GlobalVariables.ProtocolName = protocol_name
        self.assertEqual(self.study.GlobalVariables.StudyName._content, "The ODM study name")

    def test_study_dict(self):
        self.study.GlobalVariables = ODM.GlobalVariables()
        self.study.GlobalVariables.StudyName = ODM.StudyName(_content="The ODM study name")
        self.study.GlobalVariables.StudyDescription = ODM.StudyDescription(_content="The description of the ODM study")
        self.study.GlobalVariables.ProtocolName = ODM.ProtocolName(_content="The ODM protocol name")
        study_dict = self.study.to_dict()
        print(study_dict)
        self.assertDictEqual(self.expected_dict(), study_dict)

    def test_study_to_xml(self):
        self.study.GlobalVariables = ODM.GlobalVariables()
        self.study.GlobalVariables.StudyName = ODM.StudyName(_content="The ODM study name")
        self.study.GlobalVariables.StudyDescription = ODM.StudyDescription(_content="The description of the ODM study")
        self.study.GlobalVariables.ProtocolName = ODM.ProtocolName(_content="The ODM protocol name")
        study_xml = self.study.to_xml()
        gv = study_xml.find("GlobalVariables")
        pn = gv.find("ProtocolName")
        self.assertEqual(pn.text, "The ODM protocol name")

    def expected_dict(self):
        return {'OID': 'ST.001.Test', "GlobalVariables": {
                'StudyName': {'_content': 'The ODM study name'},
                'StudyDescription': {'_content': 'The description of the ODM study'},
                'ProtocolName': {'_content': 'The ODM protocol name'}}}



from unittest import TestCase
import json
import odmlib.define_2_0.model as DEFINE


class TestCodeList(TestCase):
    def setUp(self) -> None:
        attrs = self._set_attributes()
        self.cl = DEFINE.CodeList(**attrs)

    def test_add_code_list_item(self):
        cli1 = DEFINE.CodeListItem(CodedValue="HGB", ExtendedValue="Yes", OrderNumber="1")
        cli1.Alias.append(DEFINE.Alias(Context="nci:ExtCodeID", Name="C64848"))
        self.cl.CodeListItem.append(cli1)
        cli2 = DEFINE.CodeListItem(CodedValue="VITB12", OrderNumber="2")
        cli2.Alias.append(DEFINE.Alias(Context="nci:ExtCodeID", Name="C64817"))
        self.cl.CodeListItem.append(cli2)
        cli3 = DEFINE.CodeListItem(CodedValue="GLUC", OrderNumber="3")
        cli3.Alias = [DEFINE.Alias(Context="nci:ExtCodeID", Name="C41376")]
        self.cl.CodeListItem.append(cli3)
        cl_dict = self.cl.to_dict()
        print(cl_dict)
        self.assertDictEqual(cl_dict, self.expected_cli_dict())

    def test_add_enumerated_item(self):
        eni1 = DEFINE.EnumeratedItem(CodedValue="HGB", ExtendedValue="Yes", OrderNumber=1)
        eni1.Alias = [DEFINE.Alias(Context="nci:ExtCodeID", Name="C64848")]
        self.cl.EnumeratedItem.append(eni1)
        eni2 = DEFINE.EnumeratedItem(CodedValue="VITB12", OrderNumber=2)
        eni2.Alias = [DEFINE.Alias(Context="nci:ExtCodeID", Name="C64817")]
        self.cl.EnumeratedItem.append(eni2)
        eni3 = DEFINE.EnumeratedItem(CodedValue="GLUC", OrderNumber=3)
        eni3.Alias = [DEFINE.Alias(Context="nci:ExtCodeID", Name="C41376")]
        self.cl.EnumeratedItem.append(eni3)
        ei_dict = self.cl.to_dict()
        print(ei_dict)
        self.assertDictEqual(ei_dict, self.expected_eni_dict())

    def test_external_code_list(self):
        self.cl.ExternalCodeList = DEFINE.ExternalCodeList(Dictionary="MedDRA", Version="23.0", href="https://www.meddra.org/")
        ex_dict = self.cl.to_dict()
        self.assertDictEqual(ex_dict, self.expected_ex_dict())

    def test_add_description(self):
        tt1 = DEFINE.TranslatedText(_content="this is the first test description", lang="en")
        tt2 = DEFINE.TranslatedText(_content="this is the second test description", lang="en")
        desc = DEFINE.Description()
        desc.TranslatedText = [tt1, tt2]
        self.cl.Description = desc
        self.assertEqual(len(self.cl.Description.TranslatedText), 2)
        self.assertEqual(self.cl.Description.TranslatedText[1]._content, 'this is the second test description')

    def test_add_alias(self):
        alias = DEFINE.Alias(Context="nci:ExtCodeID", Name="C64848")
        self.cl.Alias = [alias]
        self.assertEqual(self.cl.Alias[0].Name, "C64848")

    def test_to_json(self):
        attrs = self._set_attributes()
        cl = DEFINE.CodeList(**attrs)
        eni = DEFINE.EnumeratedItem(CodedValue="HGB", OrderNumber=1)
        eni.Alias = [DEFINE.Alias(Context="nci:ExtCodeID", Name="C64848")]
        cl.EnumeratedItem.append(eni)
        eni.Alias.append(DEFINE.Alias(Context="nci:ExtCodeID", Name="C65047"))
        cl_json = cl.to_json()
        cl_dict = json.loads(cl_json)
        print(cl_dict)
        self.assertDictEqual(cl_dict, self.expected_json_dict())

    def test_to_dict(self):
        attrs = self._set_attributes()
        cl = DEFINE.CodeList(**attrs)
        eni = DEFINE.EnumeratedItem(CodedValue="HGB", OrderNumber=1)
        eni.Alias = [DEFINE.Alias(Context="nci:ExtCodeID", Name="C64848")]
        cl.EnumeratedItem.append(eni)
        eni.Alias.append(DEFINE.Alias(Context="nci:ExtCodeID", Name="C65047"))
        cl_dict = cl.to_dict()
        self.assertDictEqual(cl_dict, self.expected_json_dict())

    def test_to_xml(self):
        attrs = self._set_attributes()
        cl = DEFINE.CodeList(**attrs)
        eni = DEFINE.EnumeratedItem(CodedValue="HGB", OrderNumber=1)
        eni.Alias = [DEFINE.Alias(Context="nci:ExtCodeID", Name="C64848")]
        cl.EnumeratedItem.append(eni)
        eni.Alias.append(DEFINE.Alias(Context="nci:ExtCodeID", Name="C65047"))
        cl_xml = cl.to_xml()
        self.assertEqual(cl_xml.attrib["OID"], "ODM.CL.LBTESTCD")
        ei = cl_xml.find("EnumeratedItem")
        self.assertEqual(ei.attrib["CodedValue"], "HGB")

    def test_to_xml_code_list_item(self):
        attrs = self._set_attributes()
        cl = DEFINE.CodeList(**attrs)
        cli1 = DEFINE.CodeListItem(CodedValue="HGB", OrderNumber=1)
        cli1.Alias = [DEFINE.Alias(Context="nci:ExtCodeID", Name="C64848")]
        cl.CodeListItem.append(cli1)
        cli2 = DEFINE.CodeListItem(CodedValue="VITB12", OrderNumber=2)
        cli2.Alias.append(DEFINE.Alias(Context="nci:ExtCodeID", Name="C64817"))
        cl.CodeListItem.append(cli2)
        cli3 = DEFINE.CodeListItem(CodedValue="GLUC", OrderNumber=3)
        cli3.Alias.append(DEFINE.Alias(Context="nci:ExtCodeID", Name="C41376"))
        cl.CodeListItem.append(cli3)
        cl_xml = cl.to_xml()
        self.assertEqual(cl_xml.attrib["OID"], "ODM.CL.LBTESTCD")
        ei = cl_xml.findall("CodeListItem")
        self.assertEqual(ei[0].attrib["CodedValue"], "HGB")

    def test_to_xml_external_code_list(self):
        attrs = self._set_attributes()
        cl = DEFINE.CodeList(**attrs)
        excl = DEFINE.ExternalCodeList(Dictionary="MedDRA", Version="23.0", href="https://www.meddra.org/")
        cl.ExternalCodeList = excl
        cl_xml = cl.to_xml()
        self.assertEqual(cl_xml.attrib["OID"], "ODM.CL.LBTESTCD")
        ecl = cl_xml.find("ExternalCodeList")
        self.assertEqual(ecl.attrib["Dictionary"], "MedDRA")

    def _set_attributes(self):
        """
        set some CodeList element attributes using test data
        :return: dictionary with CodeList attribute information
        """
        return {"OID": "ODM.CL.LBTESTCD", "Name": "Laboratory Test Code", "DataType": "text"}

    def expected_cli_dict(self):
        return {'OID': 'ODM.CL.LBTESTCD', 'Name': 'Laboratory Test Code', 'DataType': 'text',
                'CodeListItem': [{'CodedValue': 'HGB', "ExtendedValue": "Yes", 'OrderNumber': 1,
                                  'Alias': [{'Context': 'nci:ExtCodeID', 'Name': 'C64848'}]},
                                 {'CodedValue': 'VITB12', 'OrderNumber': 2,
                                  'Alias': [{'Context': 'nci:ExtCodeID', 'Name': 'C64817'}]},
                                 {'CodedValue': 'GLUC', 'OrderNumber': 3,
                                  'Alias': [{'Context': 'nci:ExtCodeID', 'Name': 'C41376'}]}]}

    def expected_eni_dict(self):
        return {'OID': 'ODM.CL.LBTESTCD', 'Name': 'Laboratory Test Code', 'DataType': 'text',
                'EnumeratedItem': [{'CodedValue': 'HGB', "ExtendedValue": "Yes", 'OrderNumber': 1,
                                    'Alias': [{'Context': 'nci:ExtCodeID', 'Name': 'C64848'}]},
                                   {'CodedValue': 'VITB12', 'OrderNumber': 2,
                                    'Alias': [{'Context': 'nci:ExtCodeID', 'Name': 'C64817'}]},
                                   {'CodedValue': 'GLUC', 'OrderNumber': 3,
                                    'Alias': [{'Context': 'nci:ExtCodeID', 'Name': 'C41376'}]}]}

    def expected_ex_dict(self):
        return {'OID': 'ODM.CL.LBTESTCD', 'Name': 'Laboratory Test Code', 'DataType': 'text',
                'ExternalCodeList': {'Dictionary': 'MedDRA', 'Version': '23.0', 'href': 'https://www.meddra.org/'}}

    def expected_json_dict(self):
        return {'OID': 'ODM.CL.LBTESTCD', 'Name': 'Laboratory Test Code', 'DataType': 'text',
                'EnumeratedItem': [{'CodedValue': 'HGB', 'OrderNumber': 1,
                                    'Alias': [{'Context': 'nci:ExtCodeID', 'Name': 'C64848'},
                                              {'Context': 'nci:ExtCodeID', 'Name': 'C65047'}]
                                    }]
                }




from unittest import TestCase
import json
import odmlib.odm_1_3_2.model as ODM


class TestConditionDef(TestCase):
    def setUp(self) -> None:
        attrs = self.set_attributes()
        self.cond_def = ODM.ConditionDef(**attrs)

    def test_add_formal_expression(self):
        fex = ODM.FormalExpression(Context="Python 3.7", _content="BRTHYR != 4")
        self.cond_def.FormalExpression = [fex]
        self.assertEqual(self.cond_def.FormalExpression[0]._content, "BRTHYR != 4")

    def test_add_description(self):
        tt1 = ODM.TranslatedText(_content="Skip the BRTHMO field when BRTHYR length NE 4", lang="en")
        self.cond_def.Description = ODM.Description()
        self.cond_def.Description.TranslatedText = [tt1]
        self.assertEqual(len(self.cond_def.Description.TranslatedText), 1)
        self.assertEqual(self.cond_def.Description.TranslatedText[0]._content, 'Skip the BRTHMO field when BRTHYR length NE 4')

    def test_add_alias(self):
        a = ODM.Alias(Context="CDASH", Name="AGE")
        self.cond_def.Alias = [a]
        self.assertEqual(self.cond_def.Alias[0].Name, "AGE")

    def test_to_json(self):
        attrs = self.set_attributes()
        cd = ODM.ConditionDef(**attrs)
        tt1 = ODM.TranslatedText(_content="Skip the BRTHMO field when BRTHYR length NE 4", lang="en")
        cd.Description = ODM.Description()
        cd.Description.TranslatedText.append(tt1)
        cd.FormalExpression = [ODM.FormalExpression(Context="Python 3.7", _content="BRTHYR != 4")]
        cd_dict = cd.to_dict()
        print(cd_dict)
        method_json = cd.to_json()
        method_dict = json.loads(method_json)
        self.assertEqual(method_dict["OID"], "ODM.CD.BRTHMO")
        self.assertDictEqual(method_dict, self.expected_dict())

    def test_to_dict(self):
        attrs = self.set_attributes()
        cd = ODM.ConditionDef(**attrs)
        cd.Description = ODM.Description()
        cd.Description.TranslatedText = [ODM.TranslatedText(_content="Skip the BRTHMO field when BRTHYR length NE 4", lang="en")]
        cd.FormalExpression = [ODM.FormalExpression(Context="Python 3.7", _content="BRTHYR != 4")]
        cd_dict = cd.to_dict()
        print(cd_dict)
        self.assertEqual(cd_dict["OID"], "ODM.CD.BRTHMO")
        self.assertDictEqual(cd_dict, self.expected_dict())

    def test_to_xml(self):
        attrs = self.set_attributes()
        cd = ODM.ConditionDef(**attrs)
        cd.Description = ODM.Description()
        cd.Description.TranslatedText = [ODM.TranslatedText(_content="Skip the BRTHMO field when BRTHYR length NE 4", lang="en")]
        cd.FormalExpression = [ODM.FormalExpression(Context="Python 3.7", _content="BRTHYR != 4")]
        cd_xml = cd.to_xml()
        self.assertEqual(cd_xml.attrib["OID"], "ODM.CD.BRTHMO")
        fex_xml = cd_xml.find("FormalExpression")
        self.assertEqual(fex_xml.attrib["Context"], "Python 3.7")

    def set_attributes(self):
        """
        set some cond_def element attributes using test data
        :return: dictionary with cond_def attribute information
        """
        return {"OID": "ODM.CD.BRTHMO", "Name": "Skip BRTHMO when no BRTHYR"}

    def expected_dict(self):
        return {'OID': 'ODM.CD.BRTHMO', 'Name': 'Skip BRTHMO when no BRTHYR', 'Description':
            {'TranslatedText': [{'_content': 'Skip the BRTHMO field when BRTHYR length NE 4', 'lang': 'en'}]},
             'FormalExpression': [{'Context': 'Python 3.7', '_content': 'BRTHYR != 4'}]}



from unittest import TestCase
import odmlib.odm_1_3_2.model as ODM
import odmlib.odm_parser as ODM_PARSER
import os
import datetime


class TestClinicalData(TestCase):
    def setUp(self) -> None:
        self.odm_test_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data','test_clinical_data_01.xml')

    def test_clinical_data_to_xml(self):
        cd = []
        cd.append(ODM.ClinicalData(StudyOID="P2006-101", MetaDataVersionOID="101.01"))
        cd[0].SubjectData.append(ODM.SubjectData(SubjectKey="1000", TransactionType="Insert"))
        cd[0].SubjectData[0].StudyEventData.append(ODM.StudyEventData(StudyEventOID="Screen"))
        cd[0].SubjectData[0].StudyEventData[0].FormData.append(ODM.FormData(FormOID="DEMOG"))
        cd[0].SubjectData[0].StudyEventData[0].FormData[0].ItemGroupData.append(ODM.ItemGroupData(ItemGroupOID="DM"))
        cd[0].SubjectData[0].StudyEventData[0].FormData[0].ItemGroupData[0].ItemData.append(ODM.ItemData(ItemOID="USUBJID", Value="101-001-001"))
        cd[0].SubjectData[0].StudyEventData[0].FormData[0].ItemGroupData[0].ItemData.append(ODM.ItemData(ItemOID="SEX", Value="F"))
        cd[0].SubjectData[0].StudyEventData[0].FormData.append(ODM.FormData(FormOID="LABDATA"))
        cd[0].SubjectData[0].StudyEventData[0].FormData[1].ItemGroupData.append(ODM.ItemGroupData(ItemGroupOID="LB"))
        cd[0].SubjectData[0].StudyEventData[0].FormData[1].ItemGroupData[0].ItemData.append(ODM.ItemData(ItemOID="LBDTC", Value="2006-07-14T14:48"))
        cd[0].SubjectData[0].StudyEventData[0].FormData[1].ItemGroupData[0].ItemData.append(ODM.ItemData(ItemOID="LBTESTCD", Value="ALT"))
        cd[0].SubjectData[0].StudyEventData[0].FormData[1].ItemGroupData[0].ItemData.append(ODM.ItemData(ItemOID="LBORRES", Value="245"))

        cd.append(ODM.ClinicalData(StudyOID="P2006-101", MetaDataVersionOID="101.02"))
        cd[1].SubjectData.append(ODM.SubjectData(SubjectKey="1000", TransactionType="Insert"))
        cd[1].SubjectData[0].StudyEventData.append(ODM.StudyEventData(StudyEventOID="VISIT_1"))
        cd[1].SubjectData[0].StudyEventData[0].FormData.append(ODM.FormData(FormOID="AENONSER"))
        cd[1].SubjectData[0].StudyEventData[0].FormData[0].ItemGroupData.append(ODM.ItemGroupData(ItemGroupOID="AE"))
        cd[1].SubjectData[0].StudyEventData[0].FormData[0].ItemGroupData[0].ItemData.append(ODM.ItemData(ItemOID="AETERM", Value="Fever"))
        cd[1].SubjectData[0].StudyEventData[0].FormData[0].ItemGroupData[0].ItemData.append(ODM.ItemData(ItemOID="AESTDTC", Value="2006-08-21"))
        cd[1].SubjectData[0].StudyEventData[0].FormData.append(ODM.FormData(FormOID="LABDATA"))
        cd[1].SubjectData[0].StudyEventData[0].FormData[1].ItemGroupData.append(ODM.ItemGroupData(ItemGroupOID="LB"))
        cd[1].SubjectData[0].StudyEventData[0].FormData[1].ItemGroupData[0].ItemData.append(ODM.ItemData(ItemOID="LBDTC", Value="2006-07-14T14:48"))
        cd[1].SubjectData[0].StudyEventData[0].FormData[1].ItemGroupData[0].ItemData.append(ODM.ItemData(ItemOID="LBTESTCD", Value="ALT"))
        cd[1].SubjectData[0].StudyEventData[0].FormData[1].ItemGroupData[0].ItemData.append(ODM.ItemData(ItemOID="LBORRES", Value="300"))

        root = self.create_odm_document(cd)
        odm_xml = root.to_xml()
        self.assertEqual(odm_xml.attrib["FileOID"], "ODM.TEST.CD.001")
        self.assertListEqual(["ClinicalData", "ClinicalData"], [e.tag for e in odm_xml.getchildren()])

    def test_clinical_data_from_xml(self):
        parser = ODM_PARSER.ODMParser(self.odm_test_file)
        parser.parse()
        cd = parser.ClinicalData()
        subj_list = parser.SubjectData(parent=cd[1])
        sed = parser.StudyEventData(parent=subj_list[0]["elem"])
        fd = parser.FormData(parent=sed[0]["elem"])
        igd = parser.ItemGroupData(parent=fd[0]["elem"])
        item_data = parser.ItemData(parent=igd[0]["elem"])
        self.assertEqual(item_data[0]["Value"], "Fever")

    def test_audit_record(self):
        ar = ODM.AuditRecord(EditPoint="Monitoring", UsedImputationMethod="Yes", ID="1")
        ar.UserRef = ODM.UserRef(UserOID="USER.001")
        ar.LocationRef = ODM.LocationRef(LocationOID="SITE.101")
        ar.DateTimeStamp = ODM.DateTimeStamp(_content="2020-12-01T10:28:16")
        ar.ReasonForChange = ODM.ReasonForChange(_content="Missing data")
        ar.SourceID = ODM.SourceID(_content="RECID14327")
        ar_dict = ar.to_dict()
        print(ar_dict)
        self.assertEqual(ar_dict["EditPoint"], "Monitoring")
        id = ODM.ItemData(ItemOID="IT.AE.AETERM", TransactionType="Insert", Value="Fever")
        id.AuditRecord = ar
        id_dict = id.to_dict()
        self.assertEqual(id_dict["AuditRecord"]["UserRef"]["UserOID"], "USER.001")

    def test_annotation(self):
        annotation = ODM.Annotation(SeqNum="1", TransactionType="Insert", ID="ANN001")
        annotation.Comment = ODM.Comment(SponsorOrSite="Site", _content="Transferred from EHR")
        annotation.Flag.append(ODM.Flag())
        annotation.Flag[0].FlagValue = ODM.FlagValue(CodeListOID="CL.FLAGVALUE", _content="eSource")
        annotation.Flag[0].FlagType = ODM.FlagType(CodeListOID="CL.FLAGTYPE", _content="eDT")
        annotation_dict = annotation.to_dict()
        print(annotation_dict)
        self.assertEqual(annotation_dict["Flag"][0]["FlagValue"]["CodeListOID"], "CL.FLAGVALUE")
        self.assertEqual(annotation.Flag[0].FlagValue.CodeListOID, "CL.FLAGVALUE")

    def test_signature(self):
        sig = ODM.Signature(ID="SIG.001.USER.001")
        sig.UserRef = ODM.UserRef(UserOID="USER.001")
        sig.LocationRef = ODM.LocationRef(LocationOID="SITE.101")
        sig.SignatureRef = ODM.SignatureRef(SignatureOID="SIG.001")
        sig.DateTimeStamp = ODM.DateTimeStamp(_content="2020-12-01T10:28:16")
        sig_dict = sig.to_dict()
        print(sig_dict)
        self.assertEqual(sig_dict["SignatureRef"]["SignatureOID"], "SIG.001")
        self.assertEqual(sig.SignatureRef.SignatureOID, "SIG.001")

    def test_audit_records(self):
        ar = ODM.AuditRecord(EditPoint="Monitoring", UsedImputationMethod="Yes", ID="1")
        ar.UserRef = ODM.UserRef(UserOID="USER.001")
        ar.LocationRef = ODM.LocationRef(LocationOID="SITE.101")
        ar.DateTimeStamp = ODM.DateTimeStamp(_content="2020-12-01T10:28:16")
        ar.ReasonForChange = ODM.ReasonForChange(_content="Missing data")
        ar.SourceID = ODM.SourceID(_content="RECID14327")
        ars = ODM.AuditRecords()
        ars.AuditRecord.append(ar)
        self.assertEqual(ars.AuditRecord[0].ID, "1")
        self.assertEqual(ars.AuditRecord[0].LocationRef.LocationOID, "SITE.101")

    def test_signatures(self):
        sig = ODM.Signature(ID="SIG.001.USER.001")
        sig.UserRef = ODM.UserRef(UserOID="USER.001")
        sig.LocationRef = ODM.LocationRef(LocationOID="SITE.101")
        sig.SignatureRef = ODM.SignatureRef(SignatureOID="SIG.001")
        sig.DateTimeStamp = ODM.DateTimeStamp(_content="2020-12-01T10:28:16")
        sigs = ODM.Signatures()
        sigs.Signature.append(sig)
        self.assertEqual(sigs.Signature[0].ID, "SIG.001.USER.001")
        self.assertEqual(sigs.Signature[0].SignatureRef.SignatureOID, "SIG.001")

    def test_annotations(self):
        annotation = ODM.Annotation(SeqNum="1", TransactionType="Insert", ID="ANN001")
        annotation.Comment = ODM.Comment(SponsorOrSite="Site", _content="Transferred from EHR")
        annotation.Flag.append(ODM.Flag())
        annotation.Flag[0].FlagValue = ODM.FlagValue(CodeListOID="CL.FLAGVALUE", _content="eSource")
        annotation.Flag[0].FlagType = ODM.FlagType(CodeListOID="CL.FLAGTYPE", _content="eDT")
        anns = ODM.Annotations()
        anns.Annotation.append(annotation)
        self.assertEqual(anns.Annotation[0].ID, "ANN001")
        self.assertEqual(anns.Annotation[0].Flag[0].FlagValue.CodeListOID, "CL.FLAGVALUE")

    def create_odm_document(self, cd):
        """
        assemble the ODM document, add the ItemGroupDef, and write it to a file
        :param igd: ItemGroupDef object
        """
        root = self.create_root()
        for clin_data in cd:
            root.ClinicalData.append(clin_data)
        root.write_xml(self.odm_test_file)
        return root

    def create_root(self):
        """
        create the ODM root element object with test data
        :return: ODM root element object
        """
        root = {"FileOID": "ODM.TEST.CD.001", "Granularity": "Metadata", "AsOfDateTime": self.set_datetime(),
                "CreationDateTime": self.set_datetime(), "ODMVersion": "1.3.2", "Originator": "ODMLIB", "SourceSystem": "ODMLIB",
                "SourceSystemVersion": "0.1", "FileType": "Snapshot"}
        root = ODM.ODM(**root)
        return root

    def set_datetime(self):
        """return the current datetime in ISO 8601 format"""
        return datetime.datetime.utcnow().replace(tzinfo=datetime.timezone.utc).isoformat()



from unittest import TestCase
import json
import odmlib.odm_1_3_2.model as ODM
import odmlib.odm_parser as ODM_PARSER
import xml.etree.ElementTree as ET
import os
import datetime


class TestItemGroupDef(TestCase):
    def setUp(self) -> None:
        attrs = self.set_itemgroupdef_attributes()
        self.igd = ODM.ItemGroupDef(**attrs)
        self.odm_test_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data', 'test_igd_001.xml')

    def test_item_group_valid_kwargs_only(self):
        igd = ODM.ItemGroupDef(OID="IG.VS", Name="VS", Repeating="Yes")
        self.assertEqual(igd.OID, "IG.VS")

    def test_add_description(self):
        tt1 = ODM.TranslatedText(_content="this is the first test description", lang="en")
        tt2 = ODM.TranslatedText(_content="this is the second test description", lang="en")
        self.igd.Description = ODM.Description()
        self.igd.Description.TranslatedText = [tt1, tt2]
        self.assertEqual(len(self.igd.Description.TranslatedText), 2)
        self.assertEqual(self.igd.Description.TranslatedText[1]._content, 'this is the second test description')

    def test_add_item_ref(self):
        self.igd.ItemRef.append(ODM.ItemRef(ItemOID="IT.STUDYID", Mandatory="Yes", OrderNumber=1))
        self.igd.ItemRef.append(ODM.ItemRef(ItemOID="IT.VS.VSTEST", Mandatory="No", OrderNumber=2))
        self.igd.ItemRef.append(ODM.ItemRef(ItemOID="IT.VS.VSORRES", Mandatory="Yes", OrderNumber=3, MethodOID="MT.METHODFEX"))
        self.assertEqual(self.igd.ItemRef[0].ItemOID, "IT.STUDYID")
        self.assertEqual(self.igd.ItemRef[2].MethodOID, "MT.METHODFEX")

    def test_add_item_ref_list(self):
        ir1 = ODM.ItemRef(ItemOID="IT.STUDYID", Mandatory="Yes", OrderNumber=1)
        self.igd.ItemRef.append(ir1)
        ir2 = ODM.ItemRef(ItemOID="IT.VS.VSTEST", Mandatory="No", OrderNumber=2)
        self.igd.ItemRef.append(ir2)
        ir3 = ODM.ItemRef(ItemOID="IT.VS.VSORRES", Mandatory="Yes", OrderNumber=3, MethodOID="MT.METHODFEX")
        self.igd.ItemRef.append(ir3)
        self.assertEqual(self.igd.ItemRef[0].ItemOID, "IT.STUDYID")
        self.assertEqual(self.igd.ItemRef[2].MethodOID, "MT.METHODFEX")

    def test_add_item_ref_missing_kwarg(self):
        with self.assertRaises(ValueError):
            self.igd.ItemRef = [ODM.ItemRef(Mandatory="Yes", OrderNumber=1)]

    def test_add_item_ref_invalid_kwarg(self):
        with self.assertRaises(TypeError):
            self.igd.ItemRef = [ODM.ItemRef(ItemOID="IT.STUDYID", Mandatory="Yes", InValid="Yes")]

    def test_item_ref_exists(self):
        self.igd.ItemRef = [ODM.ItemRef(ItemOID="IT.VS.VSTESTCD", Mandatory="Yes", OrderNumber=4)]
        self.assertEqual(self.igd.ItemRef[0].ItemOID, "IT.VS.VSTESTCD")

    def test_add_alias(self):
        a1 = ODM.Alias(Context="SDTMIG", Name="VSORRES")
        a2 = ODM.Alias(Context="SDTMIG", Name="VSTESTCD")
        self.igd.Alias = [a1, a2]
        self.assertEqual(len(self.igd.Alias), 2)
        self.assertEqual(self.igd.Alias[1].Name, "VSTESTCD")

    def test_to_json(self):
        attrs = self.set_itemgroupdef_attributes()
        igd = ODM.ItemGroupDef(**attrs)
        tt = ODM.TranslatedText(_content="this is the first test description", lang="en")
        igd.Description = ODM.Description()
        igd.Description.TranslatedText = [tt]
        ir1 = ODM.ItemRef(ItemOID="IT.STUDYID", Mandatory="Yes", OrderNumber=1)
        ir2 = ODM.ItemRef(ItemOID="IT.VS.VSTEST", Mandatory="No", OrderNumber=2)
        igd.ItemRef = [ir1, ir2]
        igd_json = igd.to_json()
        igd_dict = json.loads(igd_json)
        print(igd_dict)
        self.assertEqual(igd_dict["OID"], "IG.VS")
        self.assertEqual(igd_dict["ItemRef"][1]["ItemOID"], "IT.VS.VSTEST")

    def test_to_xml(self):
        attrs = self.set_itemgroupdef_attributes()
        igd = ODM.ItemGroupDef(**attrs)
        tt = ODM.TranslatedText(_content="this is the first test description", lang="en")
        desc = ODM.Description()
        desc.TranslatedText = [tt]
        igd.Description = desc
        ir1 = ODM.ItemRef(ItemOID="IT.STUDYID", Mandatory="Yes", OrderNumber=1)
        ir2 = ODM.ItemRef(ItemOID="IT.VS.VSTEST", Mandatory="No", OrderNumber=2)
        igd.ItemRef = [ir1, ir2]
        igd_xml = igd.to_xml()
        self.assertEqual(igd_xml.attrib["OID"], "IG.VS")
        self.assertListEqual(["Description", "ItemRef", "ItemRef"], [e.tag for e in igd_xml.getchildren()])

    def test_itemgroupdef_round_trip(self):
        """ system test to create and serialize an ItemGroupDef object """
        attrs = self.set_itemgroupdef_attributes()
        igd = ODM.ItemGroupDef(**attrs)
        tt = ODM.TranslatedText(_content="Vital Signs", lang="en")
        igd.Description = ODM.Description()
        igd.Description.TranslatedText = [tt]
        item_refs = self.set_itemrefs()
        igd.ItemRef = item_refs
        parser = ODM_PARSER.ODMParser(self.odm_test_file)
        parser.parse()
        mdv = parser.MetaDataVersion()
        igd_list = parser.ItemGroupDef(parent=mdv[0])
        igd_dict = igd_list[0]
        self.assertEqual(igd_dict["OID"], "IG.VS")
        ir_list = parser.ItemRef(parent=igd_dict["elem"])
        # tests the __len__ in ItemGroupDef as well as the add_item_ref
        self.assertEqual(len(ir_list), len(igd))

    def test_itemgroupdef_slice(self):
        """ test the ability to reference a specific or slice of ItemRefs """
        attrs = self.set_itemgroupdef_attributes()
        igd = ODM.ItemGroupDef(**attrs)
        tt = ODM.TranslatedText(_content="Vital Signs", lang="en")
        igd.Description = ODM.Description()
        igd.Description.TranslatedText = [tt]
        item_refs = self.set_itemrefs()
        igd.ItemRef = item_refs
        parser = ODM_PARSER.ODMParser(self.odm_test_file)
        parser.parse()
        parser.MetaDataVersion()
        self.assertEqual(len(igd), 13)
        self.assertEqual(igd.ItemRef[0].ItemOID, "IT.STUDYID")
        slice_itr = igd.ItemRef[1:5]
        self.assertEqual(slice_itr[1].ItemOID, "IT.VS.USUBJID")

    def test_itemgroupdef_iterator(self):
        """ test the ability to reference a specific or slice of ItemRefs """
        attrs = self.set_itemgroupdef_attributes()
        igd = ODM.ItemGroupDef(**attrs)
        tt = ODM.TranslatedText(_content="Vital Signs", lang="en")
        igd.Description = ODM.Description()
        igd.Description.TranslatedText = [tt]
        item_refs = self.set_itemrefs()
        igd.ItemRef = item_refs
        parser = ODM_PARSER.ODMParser(self.odm_test_file)
        parser.parse()
        parser.MetaDataVersion()
        irs = [ir.ItemOID for ir in igd]
        self.assertEqual(len(irs), 13)
        self.assertEqual(len(igd), 13)

    def add_itemrefs(self, igd, item_refs):
        """
        add ItemRefs to the ItemGroupDef object
        :param igd: ItemGroupDef object
        :param item_refs: list of ItemRef dictionaries containing ItemRef attributes
        """
        for it in item_refs:
            attrs = {"ItemOID": it.oid, "Mandatory": it.mandatory, "OrderNumber": it.order_number}
            if it.key_sequence:
                attrs["KeySequence"] = it.key_sequence
            if it.method:
                attrs["MethodOID"] = it.method
            igd.add_item_ref(**attrs)

    def set_itemgroupdef_attributes(self):
        """
        set some ItemGroupDef element attributes using test data
        :return: dictionary with ItemGroupDef attribute information
        """
        return {"OID": "IG.VS", "Name": "VS", "Repeating": "Yes", "Domain": "VS", "Name": "VS", "SASDatasetName": "VS",
                "IsReferenceData": "No", "Purpose": "Tabulation"}


    def set_itemrefs(self):
        """
        set some ItemRef element attributes using test data
        :return: return a list of ItemRef named tuples
        """
        itemrefs = []
        itemrefs.append(ODM.ItemRef(ItemOID="IT.STUDYID", Mandatory="Yes", OrderNumber=1, KeySequence=1))
        itemrefs.append(ODM.ItemRef(ItemOID="IT.VS.DOMAIN", Mandatory="Yes", OrderNumber=2))
        itemrefs.append(ODM.ItemRef(ItemOID="IT.VS.USUBJID", Mandatory="Yes", OrderNumber=3, KeySequence=2, MethodOID="MT.USUBJID"))
        itemrefs.append(ODM.ItemRef(ItemOID="IT.VS.VSSEQ", Mandatory="Yes", OrderNumber=4, MethodOID="MT.SEQ"))
        itemrefs.append(ODM.ItemRef(ItemOID="IT.VS.VSTESTCD", Mandatory="Yes", OrderNumber=5, KeySequence=3))
        itemrefs.append(ODM.ItemRef(ItemOID="IT.VS.VSTEST", Mandatory="Yes", OrderNumber=6))
        itemrefs.append(ODM.ItemRef(ItemOID="IT.VS.VSPOS", Mandatory="No", OrderNumber=7))
        itemrefs.append(ODM.ItemRef(ItemOID="IT.VS.VSORRES", Mandatory="No", OrderNumber=8))
        itemrefs.append(ODM.ItemRef(ItemOID="IT.VS.VSORRESU", Mandatory="No", OrderNumber=9))
        itemrefs.append(ODM.ItemRef(ItemOID="IT.VS.VSSTRESC", Mandatory="No", OrderNumber=10, MethodOID="MT.VSSTRESC"))
        itemrefs.append(ODM.ItemRef(ItemOID="IT.VS.VSSTRESN", Mandatory="No", OrderNumber=11, MethodOID="MT.VSSTRESN"))
        itemrefs.append(ODM.ItemRef(ItemOID="IT.VS.VSSTRESU", Mandatory="No", OrderNumber=12))
        itemrefs.append(ODM.ItemRef(ItemOID="IT.VS.VSBLFL", Mandatory="No", OrderNumber=13, MethodOID="MT.VSBLFL"))
        return itemrefs


    def create_odm_document(self, igd):
        """
        assemble the ODM document, add the ItemGroupDef, and write it to a file
        :param igd: ItemGroupDef object
        """
        odm_elem = self.create_root().to_xml()
        study_elem = self.create_study().to_xml()
        odm_elem.insert(0, study_elem)
        mdv_elem = self.create_mdv().to_xml()
        study_elem.insert(1, mdv_elem)
        igd_elem = igd.to_xml()
        mdv_elem.insert(1, igd_elem)
        self.write_odm_file(odm_elem, self.odm_test_file)
        return odm_elem


    def create_root(self):
        """
        create the ODM root element object with test data
        :return: ODM root element object
        """
        root = {"xmlns:def": "http://www.cdisc.org/ns/def/v2.1", "Granularity": "Metadata", "AsOfDateTime": self.set_datetime(),
                "CreationDateTime": self.set_datetime(), "ODMVersion": "1.3.2", "Originator": "CDISC 360", "SourceSystem": "WS2",
                "SourceSystemVersion": "0.1", "def:Context": "Other"}
        root = ODM.odm_root.ODMRoot("ODM.TEST.IGD.001", **root)
        return root


    def create_study(self):
        """
        create the ODM Study object instantiated with test data
        :return: ODM Study element object
        """
        study = ODM.study.Study("ST.TEST.IGD.001")
        study.StudyName = "TEST ODM ItemGroupDef"
        study.StudyDescription = "ItemGroupDef 001"
        study.ProtocolName = "ODM ItemGroupDef"
        return study


    def create_mdv(self):
        """
        create the ODM MetaDataVersion object instantiated with test data
        :return: ODM MetaDataVersion element object
        """
        mdv = ODM.metadataversion.MetaDataVersion("MDV.TEST.IGD.001", "ItemGroupDefTest001", "ItemGroupDef Test 001",
                                                  "2.1.0")
        return mdv


    def write_odm_file(self, odm, odm_file):
        """
        write the ODM document to a file
        :param odm: ODM document
        :param odm_file: path and name of ODM file to write ODM document to
        """
        tree = ET.ElementTree(odm)
        tree.write(odm_file, xml_declaration=True)


    def set_datetime(self):
        """return the current datetime in ISO 8601 format"""
        return datetime.datetime.utcnow().replace(tzinfo=datetime.timezone.utc).isoformat()



from unittest import TestCase
import odmlib.odm_1_3_2.model as ODM
import os


class TestMetaDataVersion(TestCase):
    def setUp(self) -> None:
        attrs = self.set_attributes()
        self.mdv = ODM.MetaDataVersion(**attrs)
        self.odm_test_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data', 'test_mdv_001.xml')

    def test_add_include(self):
        inc = ODM.Include(StudyOID="STUDY.TRACE-XML-DEMO", MetaDataVersionOID="MDV.TRACE-XML-ODM-01")
        self.mdv.Include = inc
        self.assertEqual(self.mdv.Include.StudyOID, 'STUDY.TRACE-XML-DEMO')

    def test_add_protocol(self):
        self.mdv.Protocol = self.add_protocol()
        print(self.mdv.Protocol.to_dict())
        self.assertEqual(self.mdv.Protocol.StudyEventRef[1].StudyEventOID, "FOLLOW-UP")
        self.assertEqual(self.mdv.Protocol.Description.TranslatedText[0]._content, "Trace-XML Test CDASH File")

    def test_add_studyeventdef(self):
        self.mdv.Protocol = self.add_protocol()
        self.mdv.StudyEventDef = self.add_SED()
        self.assertEqual(len(self.mdv.StudyEventDef[1].FormRef), 3)
        self.assertEqual(self.mdv.StudyEventDef[1].FormRef[2].FormOID, 'ODM.F.AE')

    def test_add_formdef(self):
        self.mdv.Protocol = self.add_protocol()
        self.mdv.StudyEventDef = self.add_SED()
        self.mdv.FormDef = self.add_FD()
        self.assertEqual(len(self.mdv.FormDef[1].ItemGroupRef), 2)
        self.assertEqual(self.mdv.FormDef[0].ItemGroupRef[2].ItemGroupOID, 'ODM.IG.VS')

    def test_add_itemgroupdef(self):
        self.mdv.Protocol = self.add_protocol()
        self.mdv.StudyEventDef = self.add_SED()
        self.mdv.FormDef = self.add_FD()
        self.mdv.ItemGroupDef = self.add_IGD()
        self.assertEqual(len(self.mdv.ItemGroupDef[1].ItemRef), 2)
        self.assertEqual(self.mdv.ItemGroupDef[0].ItemRef[2].ItemOID, 'ODM.IT.VS.BP.SYSBP.VSORRES')

    def test_add_itemdef(self):
        self.mdv.Protocol = self.add_protocol()
        self.mdv.StudyEventDef = self.add_SED()
        self.mdv.FormDef = self.add_FD()
        self.mdv.ItemGroupDef = self.add_IGD()
        self.mdv.ItemDef = self.add_ITD()
        self.assertEqual("CDASH/SDTM", self.mdv.ItemDef[1].Alias[1].Context)
        self.assertEqual("Diastolic", self.mdv.ItemDef[1].Question.TranslatedText[0]._content)
        self.assertEqual("partialDate", self.mdv.ItemDef[0].DataType)

    def test_add_codelist(self):
        self.mdv.Protocol = self.add_protocol()
        self.mdv.StudyEventDef = self.add_SED()
        self.mdv.FormDef = self.add_FD()
        self.mdv.ItemGroupDef = self.add_IGD()
        self.mdv.ItemDef = self.add_ITD()
        self.mdv.CodeList = self.add_CL()
        self.assertEqual("ODM.CL.NY_SUB_Y_N", self.mdv.CodeList[0].OID)
        self.assertEqual("Yes", self.mdv.CodeList[0].CodeListItem[1].Decode.TranslatedText[0]._content)

    def test_add_methoddef(self):
        self.mdv.Protocol = self.add_protocol()
        self.mdv.StudyEventDef = self.add_SED()
        self.mdv.FormDef = self.add_FD()
        self.mdv.ItemGroupDef = self.add_IGD()
        self.mdv.ItemDef = self.add_ITD()
        self.mdv.CodeList = self.add_CL()
        self.mdv.MethodDef = self.add_MD()
        self.assertEqual("ODM.MT.DOB", self.mdv.MethodDef[0].OID)
        self.assertEqual("Concatenation of BRTHYR, BRTHMO, and BRTHDY in ISO 8601 format", self.mdv.MethodDef[0].Description.TranslatedText[0]._content)

    def test_add_conditiondef(self):
        self.mdv.Protocol = self.add_protocol()
        self.mdv.StudyEventDef = self.add_SED()
        self.mdv.FormDef = self.add_FD()
        self.mdv.ItemGroupDef = self.add_IGD()
        self.mdv.ItemDef = self.add_ITD()
        self.mdv.CodeList = self.add_CL()
        self.mdv.MethodDef = self.add_MD()
        self.mdv.ConditionDef = self.add_CD()
        self.assertEqual("ODM.CD.BRTHMO", self.mdv.ConditionDef[0].OID)
        self.assertEqual("Skip the BRTHMO field when BRTHYR length NE 4", self.mdv.ConditionDef[0].Description.TranslatedText[0]._content)

    def test_add_presentation(self):
        p = ODM.Presentation(OID="ODM.PR.CRF.DM", lang="en", _content="stylesheet")
        self.mdv.Presentation = [p]
        self.assertEqual("ODM.PR.CRF.DM", self.mdv.Presentation[0].OID)

    def test_mdv_to_dict(self):
        self.mdv.Protocol = self.add_protocol()
        self.mdv.StudyEventDef = self.add_SED()
        self.mdv.FormDef = self.add_FD()
        self.mdv.ItemGroupDef = self.add_IGD()
        self.mdv.ItemDef = self.add_ITD()
        self.mdv.CodeList = self.add_CL()
        self.mdv.MethodDef = self.add_MD()
        self.mdv.ConditionDef = self.add_CD()
        mdv_dict = self.mdv.to_dict()
        print(mdv_dict)
        self.assertDictEqual(self.expected_dict(), mdv_dict)

    def test_mdv_to_xml(self):
        self.mdv.Protocol = self.add_protocol()
        self.mdv.StudyEventDef = self.add_SED()
        self.mdv.FormDef = self.add_FD()
        self.mdv.ItemGroupDef = self.add_IGD()
        self.mdv.ItemDef = self.add_ITD()
        self.mdv.CodeList = self.add_CL()
        self.mdv.MethodDef = self.add_MD()
        self.mdv.ConditionDef = self.add_CD()
        mdv_xml = self.mdv.to_xml()
        self.assertEqual("MDV.TRACE-XML-ODM-01", mdv_xml.attrib["OID"])
        children = ['Protocol', 'StudyEventDef', 'StudyEventDef', 'FormDef', 'FormDef', 'ItemGroupDef', 'ItemGroupDef',
                    'ItemDef', 'ItemDef', 'CodeList', 'MethodDef', 'ConditionDef']
        found_list = [e.tag for e in mdv_xml.getchildren()]
        print(found_list)
        self.assertListEqual(children, [e.tag for e in mdv_xml.getchildren()])

    def add_CD(self):
        tt1 = ODM.TranslatedText(_content="Skip the BRTHMO field when BRTHYR length NE 4", lang="en")
        desc = ODM.Description()
        desc.TranslatedText = [tt1]
        cd = ODM.ConditionDef(OID="ODM.CD.BRTHMO", Name="Skip BRTHMO when no BRTHYR")
        cd.Description = desc
        return [cd]

    def add_MD(self):
        tt1 = ODM.TranslatedText(_content="Concatenation of BRTHYR, BRTHMO, and BRTHDY in ISO 8601 format", lang="en")
        desc = ODM.Description()
        desc.TranslatedText = [tt1]
        md = ODM.MethodDef(OID="ODM.MT.DOB", Name="Create BRTHDTC from date ELEMENTS", Type="Computation")
        md.Description = desc
        return [md]

    def add_CL(self):
        tt1 = ODM.TranslatedText(_content="No", lang="en")
        dc1 = ODM.Decode()
        dc1.TranslatedText = [tt1]
        cli1 = ODM.CodeListItem(CodedValue="N")
        cli1.Decode = dc1
        tt2 = ODM.TranslatedText(_content="Yes", lang="en")
        dc2 = ODM.Decode()
        dc2.TranslatedText = [tt2]
        cli2 = ODM.CodeListItem(CodedValue="Y")
        cli2.Decode = dc2
        cl = ODM.CodeList(OID="ODM.CL.NY_SUB_Y_N", Name="No Yes Response", DataType="text")
        cl.CodeListItem = [cli1, cli2]
        return [cl]


    def add_ITD(self):
        # ItemDef 1
        ttd1 = ODM.TranslatedText(_content="Date of measurements", lang="en")
        ttq1 = ODM.TranslatedText(_content="Date", lang="en")
        desc1 = ODM.Description()
        desc1.TranslatedText = [ttd1]
        q1 = ODM.Question()
        q1.TranslatedText = [ttq1]
        a1 = ODM.Alias(Context="CDASH", Name="VSDAT")
        itd1 = ODM.ItemDef(OID="ODM.IT.VS.VSDAT", Name="Date", DataType="partialDate")
        itd1.Description = desc1
        itd1.Question = q1
        itd1.Alias = [a1]
        # ItemDef 2
        ttd2 = ODM.TranslatedText(_content="Result of the vital signs measurement as originally received or collected.", lang="en")
        ttq2 = ODM.TranslatedText(_content="Diastolic", lang="en")
        desc2 = ODM.Description()
        desc2.TranslatedText = [ttd2]
        q2 = ODM.Question()
        q2.TranslatedText = [ttq2]
        a2a = ODM.Alias(Context="CDASH", Name="BP.DIABP.VSORRES")
        a2b = ODM.Alias(Context="CDASH/SDTM", Name="VSORRES+VSORRESU")
        itd2 = ODM.ItemDef(OID="ODM.IT.VS.BP.VSORRESU", Name="BP Units", DataType="text")
        itd2.Description = desc2
        itd2.Question = q2
        itd2.Alias = [a2a, a2b]
        return [itd1, itd2]

    def add_IGD(self):
        itr1 = ODM.ItemRef(ItemOID="ODM.IT.VS.VSDAT", Mandatory="Yes")
        itr2 = ODM.ItemRef(ItemOID="ODM.IT.VS.BP.DIABP.VSORRES", Mandatory="Yes")
        itr3 = ODM.ItemRef(ItemOID="ODM.IT.VS.BP.SYSBP.VSORRES", Mandatory="Yes")
        igd1 = ODM.ItemGroupDef(OID="ODM.IG.VS", Name="Vital Sign Measurement", Repeating="Yes")
        igd1.ItemRef = [itr1, itr2, itr3]
        igr4 = ODM.ItemRef(ItemOID="ODM.IT.DM.BRTHYR", Mandatory="Yes")
        igr5 = ODM.ItemRef(ItemOID="ODM.IT.DM.SEX", Mandatory="Yes")
        igd2 = ODM.ItemGroupDef(OID="ODM.IG.DM", Name="Demographics", Repeating="No")
        igd2.ItemRef = [igr4, igr5]
        return [igd1, igd2]

    def add_FD(self):
        igr1 = ODM.ItemGroupRef(ItemGroupOID="ODM.IG.Common", Mandatory="Yes", OrderNumber=1)
        igr2 = ODM.ItemGroupRef(ItemGroupOID="ODM.IG.VS_GENERAL", Mandatory="Yes", OrderNumber=2)
        igr3 = ODM.ItemGroupRef(ItemGroupOID="ODM.IG.VS", Mandatory="Yes", OrderNumber=3)
        fd1 = ODM.FormDef(OID="ODM.F.VS", Name="Vital Signs", Repeating="No")
        fd1.ItemGroupRef = [igr1, igr2, igr3]
        igr4 = ODM.ItemGroupRef(ItemGroupOID="ODM.IG.Common", Mandatory="Yes", OrderNumber=1)
        igr5 = ODM.ItemGroupRef(ItemGroupOID="ODM.IG.DM", Mandatory="Yes", OrderNumber=2)
        fd2 = ODM.FormDef(OID="ODM.F.DM", Name="Demographics", Repeating="No")
        fd2.ItemGroupRef = [igr4, igr5]
        return [fd1, fd2]

    def add_SED(self):
        fr1 = ODM.FormRef(FormOID="ODM.F.DM", Mandatory="Yes", OrderNumber=1)
        fr2 = ODM.FormRef(FormOID="ODM.F.VS", Mandatory="Yes", OrderNumber=2)
        fr3 = ODM.FormRef(FormOID="ODM.F.AE", Mandatory="Yes", OrderNumber=3)
        ser1 = ODM.StudyEventDef(OID="BASELINE", Name="Baseline Visit", Repeating="No", Type="Scheduled")
        ser1.FormRef = [fr1, fr2, fr3]
        ser2 = ODM.StudyEventDef(OID="FOLLOW-UP", Name="Follow-up Visit", Repeating="Yes", Type="Scheduled")
        ser2.FormRef = [fr1, fr2, fr3]
        return [ser1, ser2]

    def add_protocol(self):
        p = ODM.Protocol()
        tt = ODM.TranslatedText(_content="Trace-XML Test CDASH File", lang="en")
        p.Description = ODM.Description()
        p.Description.TranslatedText = [tt]
        ser1 = ODM.StudyEventRef(StudyEventOID="BASELINE", OrderNumber=1, Mandatory="Yes")
        ser2 = ODM.StudyEventRef(StudyEventOID="FOLLOW-UP", OrderNumber=2, Mandatory="Yes")
        p.StudyEventRef = [ser1, ser2]
        p.Alias = [ODM.Alias(Context="ClinicalTrials.gov", Name="trace-protocol")]
        return p

    def set_attributes(self):
        return {"OID": "MDV.TRACE-XML-ODM-01", "Name": "TRACE-XML MDV", "Description": "Trace-XML Example"}

    def expected_dict(self):
        return {
    "OID": "MDV.TRACE-XML-ODM-01",
    "Name": "TRACE-XML MDV",
    "Description": "Trace-XML Example",
    "Protocol": {
        "Description": {"TranslatedText": [{
            "_content": "Trace-XML Test CDASH File",
            "lang": "en"
        }]},
        "StudyEventRef": [
            {
                "StudyEventOID": "BASELINE",
                "OrderNumber": 1,
                "Mandatory": "Yes"
            },
            {
                "StudyEventOID": "FOLLOW-UP",
                "OrderNumber": 2,
                "Mandatory": "Yes"
            }
        ],
        "Alias": [{
            "Context": "ClinicalTrials.gov",
            "Name": "trace-protocol"
        }]
    },
    "StudyEventDef": [
        {
            "OID": "BASELINE",
            "Name": "Baseline Visit",
            "Repeating": "No",
            "Type": "Scheduled",
            "FormRef": [
                {
                    "FormOID": "ODM.F.DM",
                    "Mandatory": "Yes",
                    "OrderNumber": 1
                },
                {
                    "FormOID": "ODM.F.VS",
                    "Mandatory": "Yes",
                    "OrderNumber": 2
                },
                {
                    "FormOID": "ODM.F.AE",
                    "Mandatory": "Yes",
                    "OrderNumber": 3
                }
            ]
        },
        {
            "OID": "FOLLOW-UP",
            "Name": "Follow-up Visit",
            "Repeating": "Yes",
            "Type": "Scheduled",
            "FormRef": [
                {
                    "FormOID": "ODM.F.DM",
                    "Mandatory": "Yes",
                    "OrderNumber": 1
                },
                {
                    "FormOID": "ODM.F.VS",
                    "Mandatory": "Yes",
                    "OrderNumber": 2
                },
                {
                    "FormOID": "ODM.F.AE",
                    "Mandatory": "Yes",
                    "OrderNumber": 3
                }
            ]
        }
    ],
    "FormDef": [
        {
            "OID": "ODM.F.VS",
            "Name": "Vital Signs",
            "Repeating": "No",
            "ItemGroupRef": [
                {
                    "ItemGroupOID": "ODM.IG.Common",
                    "Mandatory": "Yes",
                    "OrderNumber": 1
                },
                {
                    "ItemGroupOID": "ODM.IG.VS_GENERAL",
                    "Mandatory": "Yes",
                    "OrderNumber": 2
                },
                {
                    "ItemGroupOID": "ODM.IG.VS",
                    "Mandatory": "Yes",
                    "OrderNumber": 3
                }
            ]
        },
        {
            "OID": "ODM.F.DM",
            "Name": "Demographics",
            "Repeating": "No",
            "ItemGroupRef": [
                {
                    "ItemGroupOID": "ODM.IG.Common",
                    "Mandatory": "Yes",
                    "OrderNumber": 1
                },
                {
                    "ItemGroupOID": "ODM.IG.DM",
                    "Mandatory": "Yes",
                    "OrderNumber": 2
                }
            ]
        }
    ],
    "ItemGroupDef": [
        {
            "OID": "ODM.IG.VS",
            "Name": "Vital Sign Measurement",
            "Repeating": "Yes",
            "ItemRef": [
                {
                    "ItemOID": "ODM.IT.VS.VSDAT",
                    "Mandatory": "Yes"
                },
                {
                    "ItemOID": "ODM.IT.VS.BP.DIABP.VSORRES",
                    "Mandatory": "Yes"
                },
                {
                    "ItemOID": "ODM.IT.VS.BP.SYSBP.VSORRES",
                    "Mandatory": "Yes"
                }
            ]
        },
        {
            "OID": "ODM.IG.DM",
            "Name": "Demographics",
            "Repeating": "No",
            "ItemRef": [
                {
                    "ItemOID": "ODM.IT.DM.BRTHYR",
                    "Mandatory": "Yes"
                },
                {
                    "ItemOID": "ODM.IT.DM.SEX",
                    "Mandatory": "Yes"
                }
            ]
        }
    ],
    "ItemDef": [
        {
            "OID": "ODM.IT.VS.VSDAT",
            "Name": "Date",
            "DataType": "partialDate",
            "Description": {"TranslatedText": [{
                "_content": "Date of measurements",
                "lang": "en"
            }]},
            "Question": {"TranslatedText": [{
                "_content": "Date",
                "lang": "en"
            }]},
            "Alias": [{
                "Context": "CDASH",
                "Name": "VSDAT"
            }]
        },
        {
            "OID": "ODM.IT.VS.BP.VSORRESU",
            "Name": "BP Units",
            "DataType": "text",
            "Description": {"TranslatedText": [{
                "_content": "Result of the vital signs measurement as originally received or collected.",
                "lang": "en"
            }]},
            "Question": {"TranslatedText": [{
                "_content": "Diastolic",
                "lang": "en"
            }]},
            "Alias": [
                {
                    "Context": "CDASH",
                    "Name": "BP.DIABP.VSORRES"
                },
                {
                    "Context": "CDASH/SDTM",
                    "Name": "VSORRES+VSORRESU"
                }
            ]
        }
    ],
    "CodeList": [{
        "OID": "ODM.CL.NY_SUB_Y_N",
        "Name": "No Yes Response",
        "DataType": "text",
        "CodeListItem": [
            {
                "CodedValue": "N",
                "Decode": {"TranslatedText": [{
                    "_content": "No",
                    "lang": "en"
                }]}
            },
            {
                "CodedValue": "Y",
                "Decode": {"TranslatedText": [{
                    "_content": "Yes",
                    "lang": "en"
                }]}
            }
        ]
    }],
    "MethodDef": [{
        "OID": "ODM.MT.DOB",
        "Name": "Create BRTHDTC from date ELEMENTS",
        "Type": "Computation",
        "Description": {"TranslatedText": [{
            "_content": "Concatenation of BRTHYR, BRTHMO, and BRTHDY in ISO 8601 format",
            "lang": "en"
        }]}
    }],
    "ConditionDef": [{
        "OID": "ODM.CD.BRTHMO",
        "Name": "Skip BRTHMO when no BRTHYR",
        "Description": {"TranslatedText": [{
            "_content": "Skip the BRTHMO field when BRTHYR length NE 4",
            "lang": "en"
        }]}
    }]
}



from unittest import TestCase
import odmlib.define_loader as OL
import odmlib.loader as LD
import os


class TestODMLoader(TestCase):
    def setUp(self) -> None:
        self.odm_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data', 'define2-0-0-sdtm-test.xml')
        self.odm_file_json = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data', 'sdtm_define2_test.json')
        self.loader = LD.ODMLoader(OL.XMLDefineLoader())
        self.jloader = LD.ODMLoader(OL.JSONDefineLoader())

    def test_unknown_loader(self):
        with self.assertRaises(TypeError):
            loader = LD.ODMLoader(self.test_mdv_find_by_OID())

    def test_open_odm_document(self):
        root = self.loader.open_odm_document(self.odm_file)
        elem_name = root.tag[root.tag.find('}') + 1:]
        self.assertEqual("ODM", elem_name)
        self.assertEqual("www.cdisc.org.Studycdisc01-Define-XML_2.0.0", root.attrib["FileOID"])

    def test_load_odm(self):
        root = self.loader.open_odm_document(self.odm_file)
        odm = self.loader.load_odm()
        self.assertEqual("www.cdisc.org.Studycdisc01-Define-XML_2.0.0", odm.FileOID)

    def test_load_missing(self):
        root = self.loader.open_odm_document(self.odm_file)
        with self.assertRaises(AttributeError):
            odm = self.loader.load_standard()

    def test_load_study(self):
        root = self.loader.open_odm_document(self.odm_file)
        study = self.loader.load_study()
        self.assertEqual("cdisc01", study.OID)

    def test_load_doc_from_string_json(self):
        with open(self.odm_file_json, 'r') as file:
            json_string = file.read()
        odm = self.jloader.create_document_from_string(json_string)
        self.assertEqual("www.cdisc.org.Studycdisc01-Define-XML_2.0.0", odm["FileOID"])

    def test_load_odm_json(self):
        root = self.jloader.open_odm_document(self.odm_file_json)
        odm = self.jloader.load_odm()
        self.assertEqual("www.cdisc.org.Studycdisc01-Define-XML_2.0.0", odm.FileOID)

    def test_load_study_json(self):
        root = self.jloader.open_odm_document(self.odm_file_json)
        study = self.jloader.load_study()
        self.assertEqual("cdisc01", study.OID)
        study = self.jloader.Study()
        self.assertEqual("cdisc01", study.OID)

    def test_meta_data_version(self):
        self.loader.open_odm_document(self.odm_file)
        mdv = self.loader.MetaDataVersion()
        self.assertEqual(mdv.Name, "Study CDISC01, Data Definitions")
        self.assertEqual(mdv.ValueListDef[0].OID, "VL.DA.DAORRES")
        self.assertEqual(mdv.ItemDef[0].CodeListRef.CodeListOID, "CL.ACN")
        self.assertEqual(mdv.CodeList[1].CodeListItem[0].CodedValue, "AE")
        self.assertEqual(mdv.CommentDef[0].OID, "COM.AGEU")

    def test_meta_data_version_json(self):
        self.jloader.open_odm_document(self.odm_file_json)
        mdv = self.jloader.load_metadataversion()
        self.assertEqual(mdv.Name, "Study CDISC01, Data Definitions")
        self.assertEqual(mdv.ValueListDef[0].OID, "VL.DA.DAORRES")
        self.assertEqual(mdv.ItemDef[0].CodeListRef.CodeListOID, "CL.ACN")
        self.assertEqual(mdv.CodeList[1].CodeListItem[0].CodedValue, "AE")
        self.assertEqual(mdv.CommentDef[0].OID, "COM.AGEU")

    def test_mdv_find_by_OID(self):
        self.loader.open_odm_document(self.odm_file)
        mdv = self.loader.MetaDataVersion()
        self.assertEqual(mdv.Name, "Study CDISC01, Data Definitions")
        ir = mdv.ItemGroupDef[0].find("ItemRef", "ItemOID", "IT.TA.DOMAIN")
        self.assertEqual(ir.ItemOID, mdv.ItemGroupDef[0].ItemRef[1].ItemOID)
        it = mdv.find("ItemDef", "OID", "IT.AE.AESEV")
        self.assertEqual(it.CodeListRef.CodeListOID, "CL.AESEV")
        cli = mdv.CodeList[4].find("CodeListItem", "CodedValue", "MODERATE")
        self.assertEqual(cli.Decode.TranslatedText[0]._content, "Grade 2")
        ir = mdv.ValueListDef[2].find("ItemRef", "ItemOID", "IT.EG.EGSTRESC.QTCF")
        self.assertEqual(ir.WhereClauseRef[0].WhereClauseOID, "WC.EG.EGTESTCD.QTCF")

    def test_odm_round_trip(self):
        root = self.loader.open_odm_document(self.odm_file)
        odm = self.loader.create_odmlib(root)
        odm_json = odm.to_json()
        odm_json_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data', 'sdtm_define2_test.json')
        with open(odm_json_file, "w") as odm_in:
            odm_in.write(odm_json)
        json_loader = LD.ODMLoader(OL.JSONDefineLoader())
        odm_dict = json_loader.open_odm_document(odm_json_file)
        rt_odm = json_loader.create_odmlib(odm_dict, "ODM")
        def_xml_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data', 'sdtm_def_test_roundtrip.xml')
        rt_odm.write_xml(def_xml_file)
        root2 = self.loader.open_odm_document(def_xml_file)
        odm2 = self.loader.create_odmlib(root2)
        self.assertEqual(odm2.Study[0].MetaDataVersion.OID, "MDV.CDISC01.SDTMIG.3.1.2.SDTM.1.2")
        print(f"ItemRefs = {len(odm2.Study[0].MetaDataVersion.ItemGroupDef[1])}")
        self.assertEqual(len(odm2.Study[0].MetaDataVersion.ItemGroupDef[1]), 6)


import odmlib.document_loader as DL
import odmlib.odm_parser as P
#import odmlib.odm_1_3_2.model as ODM
import odmlib.ns_registry as NS
import json
import importlib
import xml.etree.ElementTree as ET

ODM_PREFIX = "odm:"
ODM_NS = {'odm': 'http://www.cdisc.org/ns/odm/v1.3'}


class JSONODMLoader(DL.DocumentLoader):
    def __init__(self, model_package="odm_1_3_2"):
        self.filename = None
        self.odm_dict = {}
        self.ODM = importlib.import_module(f"odmlib.{model_package}.model")

    def load_document(self, odm_dict, key):
        attrib = {key: value for key, value in odm_dict.items() if not isinstance(value, (list, dict))}
        odm_obj = eval("self.ODM." + key + "(**" + str(attrib) + ")")
        odm_obj_items = eval("self.ODM." + key + ".__dict__.items()")
        for k, v in odm_obj_items:
            if type(v).__name__ == "ODMObject":
                if k in odm_dict:
                    odm_child_obj = self.load_document(odm_dict[k], k)
                    exec("odm_obj." + k + " = odm_child_obj")
            elif type(v).__name__ == "ODMListObject":
                if k in odm_dict:
                    for val in odm_dict[k]:
                        odm_child_obj = self.load_document(val, k)
                        eval("odm_obj." + k + ".append(odm_child_obj)")
        return odm_obj

    def create_document(self, filename):
        self.filename = filename
        with open(self.filename) as json_in:
            self.odm_dict = json.load(json_in)
        return self.odm_dict

    def create_document_from_string(self, odm_string):
        self.odm_dict = json.loads(odm_string)
        return self.odm_dict

    def load_odm(self):
        if not self.odm_dict:
            raise ValueError("create_document must be used to creat the document before executing load_odm")
        odm_odmlib = self.load_document(self.odm_dict, "MetaDataVersion")
        return odm_odmlib

    def load_metadataversion(self, idx=0):
        if not self.odm_dict:
            raise ValueError("create_document must be used to create the document before executing load_metadataversion")
        mdv_dict = self.odm_dict["Study"][0]["MetaDataVersion"][idx]
        mdv_odmlib = self.load_document(mdv_dict, "MetaDataVersion")
        return mdv_odmlib

    def load_study(self, idx=0):
        if not self.odm_dict:
            raise ValueError("create_document must be used to create the document before executing load_metadataversion")
        study_dict = self.odm_dict["Study"][idx]
        study_odmlib = self.load_document(study_dict, "Study")
        return study_odmlib


class DictODMLoader(JSONODMLoader):
    pass


class XMLODMLoader(DL.DocumentLoader):
    def __init__(self, model_package="odm_1_3_2", ns_uri="http://www.cdisc.org/ns/odm/v1.3"):
        self.filename = None
        self.parser = None
        self.ODM = importlib.import_module(f"odmlib.{model_package}.model")
        self.nsr = NS.NamespaceRegistry(prefix="odm", uri=ns_uri, is_default=True)

    def load_document(self, elem, *args):
        elem_name = elem.tag[elem.tag.find('}') + 1:]
        #ns = elem.tag[:elem.tag.find('}') + 1]
        #prefix, namespace = self.nsr.get_prefix_ns_from_uri(elem.tag[:elem.tag.find('}') + 1])
        if elem.text and not elem.text.isspace():
            attrib = {**elem.attrib, **{"_content": elem.text}}
            odm_obj = eval("self.ODM." + elem_name + "(**" + str(attrib) + ")")
        else:
            odm_obj = eval("self.ODM." + elem_name + "(**" + str(elem.attrib) + ")")
        odm_obj_dict = eval("self.ODM." + elem_name + ".__dict__.items()")
        for k, v in odm_obj_dict:
            if type(v).__name__ == "ODMObject":
                namespace = self.nsr.get_ns_entry_dict(v.namespace)
                e = elem.find(v.namespace + ":" + k, namespace)
                if e is not None:
                    odm_child_obj = self.load_document(e)
                    exec("odm_obj." + k + " = odm_child_obj")
            elif type(v).__name__ == "ODMListObject":
                namespace = self.nsr.get_ns_entry_dict(v.namespace)
                for e in elem.findall(v.namespace + ":" + k, namespace):
                    odm_child_obj = self.load_document(e)
                    eval("odm_obj." + k + ".append(odm_child_obj)")
        return odm_obj

    def create_document(self, filename, namespace_registry=None):
        self.filename = filename
        self._set_namespace(namespace_registry)
        self.parser = P.ODMParser(self.filename, self.nsr)
        root = self.parser.parse()
        return root

    def create_document_from_string(self, odm_string, namespace_registry=None):
        self._set_namespace(namespace_registry)
        self.parser = P.ODMStringParser(odm_string, self.nsr)
        root = self.parser.parse()
        return root

    def _set_namespace(self, namespace_registry):
        if namespace_registry:
            self.nsr = namespace_registry
        else:
            self.nsr = NS.NamespaceRegistry(prefix="odm", uri="http://www.cdisc.org/ns/odm/v1.3", is_default=True)

    def load_odm(self):
        root = self.parser.ODM()
        root_odmlib = self.load_document(root)
        return root_odmlib

    def load_metadataversion(self, idx=0):
        mdv = self.parser.MetaDataVersion()
        mdv_odmlib = self.load_document(mdv[idx])
        return mdv_odmlib

    def load_study(self, idx=0):
        study = self.parser.Study()
        study_odmlib = self.load_document(study[idx])
        return study_odmlib



from unittest import TestCase
import odmlib.odm_1_3_2.model as ODM
import json


class TestRangeCheck(TestCase):
    def setUp(self) -> None:
        attrs = self._set_attributes()
        self.range_check = ODM.RangeCheck(**attrs)

    def test_add_check_value(self):
        self.range_check.CheckValue = [ODM.CheckValue(_content="DIABP")]
        self.assertEqual(self.range_check.CheckValue[0]._content, "DIABP")

    def test_add_formal_expression(self):
        self.range_check.FormalExpression = [ODM.FormalExpression(Context="Python 3.7", _content="print('hello world')")]
        self.assertEqual(self.range_check.FormalExpression[0]._content, "print('hello world')")

    def test_add_measurement_unit_ref(self):
        self.range_check.MeasurementUnitRef = ODM.MeasurementUnitRef(MeasurementUnitOID="ODM.MU.UNITS")
        self.assertEqual(self.range_check.MeasurementUnitRef.MeasurementUnitOID, "ODM.MU.UNITS")

    def test_add_error_message(self):
        #self.range_check.ErrorMessage = ODM.ErrorMessage()
        self.range_check.ErrorMessage.TranslatedText.append(ODM.TranslatedText(_content="invalid test code", lang="en"))
        self.range_check.ErrorMessage.TranslatedText.append(ODM.TranslatedText(_content="code de test invalide", lang="fr"))
        self.assertEqual(len(self.range_check.ErrorMessage.TranslatedText), 2)
        self.assertEqual(self.range_check.ErrorMessage.TranslatedText[1]._content, 'code de test invalide')

    def test_to_json(self):
        attrs = self._set_attributes()
        rc = ODM.RangeCheck(**attrs)
        rc.CheckValue = [ODM.CheckValue(_content="DIABP")]
        rc.FormalExpression = [ODM.FormalExpression(Context="Python 3.7", _content="print('hello world')")]
        rc.MeasurementUnitRef = ODM.MeasurementUnitRef(MeasurementUnitOID="ODM.MU.UNITS")
        # commented out when allowed empty wrapper ELEMENTS to be virtually instantiated (may not keep this)
        #rc.ErrorMessage = ODM.ErrorMessage()
        rc.ErrorMessage.TranslatedText.append(ODM.TranslatedText(_content="invalid test code", lang="en"))
        rc.ErrorMessage.TranslatedText.append(ODM.TranslatedText(_content="code de test invalide", lang="fr"))
        rc_json = rc.to_json()
        rc_dict = json.loads(rc_json)
        print(rc_dict)
        self.assertDictEqual(rc_dict, self._json_range_check_dict())

    def test_to_dict(self):
        attrs = self._set_attributes()
        rc = ODM.RangeCheck(**attrs)
        rc.CheckValue = [ODM.CheckValue(_content="DIABP")]
        rc.FormalExpression = [ODM.FormalExpression(Context="Python 3.7", _content="print('hello world')")]
        rc.MeasurementUnitRef = ODM.MeasurementUnitRef(MeasurementUnitOID="ODM.MU.UNITS")
        rc.ErrorMessage.TranslatedText.append(ODM.TranslatedText(_content="invalid test code", lang="en"))
        rc.ErrorMessage.TranslatedText.append(ODM.TranslatedText(_content="code de test invalide", lang="fr"))
        rc_dict = rc.to_dict()
        print(rc_dict)
        rc_dict = rc.to_dict()
        self.assertDictEqual(rc_dict, self._json_range_check_dict())

    def test_to_xml(self):
        attrs = self._set_attributes()
        rc = ODM.RangeCheck(**attrs)
        rc.CheckValue = [ODM.CheckValue(_content="DIABP")]
        rc.FormalExpression = [ODM.FormalExpression(Context="Python 3.7", _content="print('hello world')")]
        rc.MeasurementUnitRef = ODM.MeasurementUnitRef(MeasurementUnitOID="ODM.MU.UNITS")
        #rc.ErrorMessage = ODM.ErrorMessage()
        rc.ErrorMessage.TranslatedText.append(ODM.TranslatedText(_content="invalid test code", lang="en"))
        rc.ErrorMessage.TranslatedText.append(ODM.TranslatedText(_content="code de test invalide", lang="fr"))
        rc_xml = rc.to_xml()
        self.assertEqual(rc_xml.attrib["SoftHard"], "Soft")

    def _set_attributes(self):
        """
        set some RangeCheck element attributes using test data
        :return: dictionary with RangeCheck attribute information
        """
        return {"Comparator": "EQ", "SoftHard": "Soft"}

    def _json_range_check_dict(self):
        return {'Comparator': 'EQ', 'SoftHard': 'Soft', 'CheckValue': [{'_content': 'DIABP'}], 'FormalExpression':
            [{'Context': 'Python 3.7', '_content': "print('hello world')"}],
            'MeasurementUnitRef': {'MeasurementUnitOID': 'ODM.MU.UNITS'},
            'ErrorMessage': {'TranslatedText': [{'lang': 'en', '_content': 'invalid test code'},
                             {'lang': 'fr', '_content': 'code de test invalide'}]}
                }



from unittest import TestCase
import odmlib.define_2_0.model as DEFINE
import xml.etree.ElementTree as ET


class TestValueListDef(TestCase):
    def test_value_list_def(self):
        vld = DEFINE.ValueListDef(OID="VL.DA.DAORRES")
        vld.ItemRef.append(DEFINE.ItemRef(ItemOID="IT.DA.DAORRES.DISPAMT", OrderNumber="1", Mandatory="Yes"))
        vld.ItemRef[0].WhereClauseRef.append(DEFINE.WhereClauseRef(WhereClauseOID="WC.DA.DATESTCD.DISPAMT"))
        vld.ItemRef.append(DEFINE.ItemRef(ItemOID="IT.DA.DAORRES.RETAMT", OrderNumber="2", Mandatory="No"))
        vld.ItemRef[1].WhereClauseRef.append(DEFINE.WhereClauseRef(WhereClauseOID="WC.DA.DATESTCD.RETAMT"))
        self.assertEqual("WC.DA.DATESTCD.RETAMT", vld.ItemRef[1].WhereClauseRef[0].WhereClauseOID)
        vld_dict = vld.to_dict()
        print(vld_dict)
        expected_dict = {'OID': 'VL.DA.DAORRES', 'ItemRef':
            [{'ItemOID': 'IT.DA.DAORRES.DISPAMT', 'OrderNumber': 1, 'Mandatory': 'Yes', 'WhereClauseRef':
                [{'WhereClauseOID': 'WC.DA.DATESTCD.DISPAMT'}]},
             {'ItemOID': 'IT.DA.DAORRES.RETAMT', 'OrderNumber': 2, 'Mandatory': 'No', 'WhereClauseRef':
                 [{'WhereClauseOID': 'WC.DA.DATESTCD.RETAMT'}]}]}
        self.assertDictEqual(vld_dict, expected_dict)

    def test_where_clause_def(self):
        wcd = DEFINE.WhereClauseDef(OID="WC.DA.DATESTCD.DISPAMT")
        wcd.RangeCheck.append(DEFINE.RangeCheck(SoftHard="Soft", ItemOID="IT.DA.DATESTCD", Comparator="EQ"))
        wcd.RangeCheck[0].CheckValue.append(DEFINE.CheckValue(_content="DISPAMT"))
        self.assertEqual("IT.DA.DATESTCD", wcd.RangeCheck[0].ItemOID)
        wcd_xml = wcd.to_xml()
        print(ET.tostring(wcd_xml, encoding='utf8', method='xml'))
        self.assertDictEqual({"OID": "WC.DA.DATESTCD.DISPAMT"}, wcd_xml.attrib)

    def test_leaf(self):
        leaf = DEFINE.leaf(ID="LF.blankcrf", href="blankcrf.pdf")
        leaf.title = DEFINE.title(_content="Annotated Case Report Form")
        self.assertEqual("LF.blankcrf", leaf.ID)
        leaf_dict = leaf.to_dict()
        print(leaf_dict)
        expected_dict = {'ID': 'LF.blankcrf', 'href': 'blankcrf.pdf', 'title':
            {'_content': 'Annotated Case Report Form'}}
        self.assertDictEqual(leaf_dict, expected_dict)

    def test_comment_def(self):
        comment = DEFINE.CommentDef(OID="COM.VS.VISITNUM")
        comment.Description.TranslatedText.append(DEFINE.TranslatedText(_content="Assigned from the TV domain based on the VISIT", lang="en"))
        self.assertEqual("COM.VS.VISITNUM", comment.OID)



from unittest import TestCase
import odmlib.odm_loader as OL
import odmlib.loader as LD
import os


class TestODMLoaderString(TestCase):
    def setUp(self) -> None:
        self.odm_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data', 'cdash-odm-test.xml')
        with open(self.odm_file, "r", encoding="utf-8") as f:
            self.odm_string = f.read()
        self.loader = LD.ODMLoader(OL.XMLODMLoader())

    def test_open_odm_document(self):
        root = self.loader.load_odm_string(self.odm_string)
        elem_name = root.tag[root.tag.find('}') + 1:]
        self.assertEqual("ODM", elem_name)
        self.assertEqual("CDASH_File_2011-10-24", root.attrib["FileOID"])

    def test_odm(self):
        self.loader.load_odm_string(self.odm_string)
        odm = self.loader.root()
        self.assertEqual(odm.FileOID, "CDASH_File_2011-10-24")

    def test_meta_data_version(self):
        self.loader.load_odm_string(self.odm_string)
        mdv = self.loader.MetaDataVersion()
        self.assertEqual(mdv.Name, "TRACE-XML MDV")
        self.assertEqual(mdv.Protocol.StudyEventRef[0].StudyEventOID, "BASELINE")
        self.assertEqual(mdv.ItemDef[3].Question.TranslatedText[0]._content, "Visit Date")
        self.assertEqual(mdv.CodeList[0].CodeListItem[1].CodedValue, "MODERATE")

    def test_mdv_find_by_OID(self):
        self.loader.load_odm_string(self.odm_string)
        mdv = self.loader.MetaDataVersion()
        self.assertEqual(mdv.Name, "TRACE-XML MDV")
        ir = mdv.ItemGroupDef[0].find("ItemRef", "ItemOID", "ODM.IT.Common.SiteID")
        self.assertEqual(ir.ItemOID, mdv.ItemGroupDef[0].ItemRef[1].ItemOID)
        it = mdv.find("ItemDef", "OID", "ODM.IT.AE.AEYN")
        self.assertEqual(it.CodeListRef.CodeListOID, "ODM.CL.NY_SUB_Y_N")
        cli = mdv.CodeList[2].find("CodeListItem", "CodedValue", "DOSE REDUCED")
        self.assertEqual(cli.Decode.TranslatedText[0]._content, "DOSE REDUCED")

    def test_odm_round_trip(self):
        root = self.loader.load_odm_string(self.odm_string)
        odm = self.loader.create_odmlib(root)
        odm_json = odm.to_json()
        odm_json_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data', 'cdash_odm_test.json')
        with open(odm_json_file, "w") as odm_in:
            odm_in.write(odm_json)
        json_loader = LD.ODMLoader(OL.JSONODMLoader())
        odm_dict = json_loader.open_odm_document(odm_json_file)
        rt_odm = json_loader.create_odmlib(odm_dict, "ODM")
        odm_xml_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data', 'cdash_odm_test_roundtrip.xml')
        rt_odm.write_xml(odm_xml_file)
        root2 = self.loader.open_odm_document(odm_xml_file)
        odm2 = self.loader.create_odmlib(root2)
        self.assertEqual(odm2.Study[0].MetaDataVersion[0].ItemDef[0].OID, "ODM.IT.Common.StudyID")
        self.assertEqual(len(odm2.Study[0].MetaDataVersion[0].ItemGroupDef[1]), 7)


import odmlib.odm_1_3_2.model as ODM
import odmlib.odm_element as OE
import odmlib.typed as T
import odmlib.ns_registry as NS


NS.NamespaceRegistry(prefix="def", uri="http://www.cdisc.org/ns/def/v2.0")


class TranslatedText(ODM.TranslatedText):
    lang = ODM.TranslatedText.lang
    _content = ODM.TranslatedText._content


class Alias(ODM.Alias):
    Context = ODM.Alias.Context
    Name = ODM.Alias.Name


class StudyDescription(ODM.StudyDescription):
    _content = ODM.StudyDescription._content


class ProtocolName(ODM.ProtocolName):
    _content = ODM.ProtocolName._content


class StudyName(ODM.StudyName):
    _content = ODM.StudyName._content


class GlobalVariables(ODM.GlobalVariables):
    StudyName = ODM.GlobalVariables.StudyName
    StudyDescription = ODM.GlobalVariables.StudyDescription
    ProtocolName = ODM.GlobalVariables.ProtocolName


class Description(ODM.Description):
    TranslatedText = ODM.Description.TranslatedText


class WhereClauseRef(OE.ODMElement):
    namespace = "def"
    WhereClauseOID = T.OIDRef(required=True)


class ItemRef(ODM.ItemRef):
    ItemOID = ODM.ItemRef.ItemOID
    OrderNumber = ODM.ItemRef.OrderNumber
    Mandatory = ODM.ItemRef.Mandatory
    KeySequence = ODM.ItemRef.KeySequence
    MethodOID = ODM.ItemRef.MethodOID
    Role = ODM.ItemRef.Role
    RoleCodeListOID = ODM.ItemRef.RoleCodeListOID
    WhereClauseRef = T.ODMListObject(required=True, element_class=WhereClauseRef, namespace="def")


class title(OE.ODMElement):
    namespace = "def"
    _content = T.String(required=True)


class leaf(OE.ODMElement):
    namespace = "def"
    ID = T.ID(required=True)
    href = T.String(required=True, namespace="xlink")
    title = T.ODMObject(required=True, element_class=title, namespace="def")


class ItemGroupDef(ODM.ItemGroupDef):
    OID = ODM.ItemGroupDef.OID
    Name = ODM.ItemGroupDef.Name
    Repeating = ODM.ItemGroupDef.Repeating
    IsReferenceData = ODM.ItemGroupDef.IsReferenceData
    SASDatasetName = ODM.ItemGroupDef.SASDatasetName
    Domain = ODM.ItemGroupDef.Domain
    Purpose = ODM.ItemGroupDef.Purpose
    Structure = T.String(required=True, namespace="def")
    Class = T.ValueSetString(required=True, namespace="def")
    ArchiveLocationID = T.String(required=True, namespace="def")
    CommentOID = T.OIDRef(namespace="def")
    Description = ODM.ItemGroupDef.Description
    ItemRef = ODM.ItemGroupDef.ItemRef
    Alias = ODM.ItemGroupDef.Alias
    leaf = T.ODMObject(required=True, element_class=leaf, namespace="def")

    def __len__(self):
        return len(self.ItemRef)

    def __getitem__(self, position):
        return self.ItemRef[position]

    def __iter__(self):
        return iter(self.ItemRef)


class CheckValue(ODM.CheckValue):
    _content = ODM.CheckValue._content


class FormalExpression(ODM.FormalExpression):
    Context = ODM.FormalExpression.Context
    _content = ODM.FormalExpression._content


class RangeCheck(ODM.RangeCheck):
    Comparator = ODM.RangeCheck.Comparator
    SoftHard = ODM.RangeCheck.SoftHard
    ItemOID = T.OIDRef(required=True, namespace="def")
    CheckValue = T.ODMListObject(element_class=CheckValue)


class CodeListRef(ODM.CodeListRef):
    CodeListOID = ODM.CodeListRef.CodeListOID


class ValueListRef(OE.ODMElement):
    namespace = "def"
    ValueListOID = T.OIDRef(required=True)


class PDFPageRef(OE.ODMElement):
    namespace = "def"
    Type = T.ValueSetString(required=True)
    PageRefs = T.String()
    FirstPage = T.PositiveInteger()
    LastPage = T.PositiveInteger()


class DocumentRef(OE.ODMElement):
    namespace = "def"
    leafID = T.String(required=True)
    PDFPageRef = T.ODMListObject(element_class=PDFPageRef, namespace="def")


class Origin(OE.ODMElement):
    namespace = "def"
    Type = T.ValueSetString(required=True)
    DocumentRef = T.ODMListObject(element_class=DocumentRef, namespace="def")
    Description = T.ODMObject(element_class=Description)


class ItemDef(ODM.ItemDef):
    OID = ODM.ItemDef.OID
    Name = ODM.ItemDef.Name
    DataType = ODM.ItemDef.DataType
    Length = ODM.ItemDef.Length
    SignificantDigits = ODM.ItemDef.SignificantDigits
    SASFieldName = ODM.ItemDef.SASFieldName
    DisplayFormat = T.String(namespace="def")
    CommentOID = T.OIDRef(namespace="def")
    Description = ODM.ItemDef.Description
    CodeListRef = ODM.ItemDef.CodeListRef
    Origin = T.ODMObject(element_class=Origin, namespace="def")
    ValueListRef = T.ODMObject(element_class=ValueListRef, namespace="def")
    Alias = T.ODMListObject(element_class=Alias)


class Decode(ODM.Decode):
    TranslatedText = ODM.Decode.TranslatedText


class CodeListItem(ODM.CodeListItem):
    CodedValue = ODM.CodeListItem.CodedValue
    Rank = ODM.CodeListItem.Rank
    OrderNumber = ODM.CodeListItem.OrderNumber
    ExtendedValue = T.ValueSetString(namespace="def")
    Decode = ODM.CodeListItem.Decode
    Alias = ODM.CodeListItem.Alias


class EnumeratedItem(ODM.EnumeratedItem):
    CodedValue = ODM.EnumeratedItem.CodedValue
    Rank = ODM.EnumeratedItem.Rank
    OrderNumber = ODM.EnumeratedItem.OrderNumber
    ExtendedValue = T.String(namespace="def")
    Alias = ODM.EnumeratedItem.Alias


class ExternalCodeList(ODM.ExternalCodeList):
    Dictionary = ODM.ExternalCodeList.Dictionary
    Version = ODM.ExternalCodeList.Version
    ref = ODM.ExternalCodeList.ref
    href = ODM.ExternalCodeList.href


class CodeList(ODM.CodeList):
    OID = ODM.CodeList.OID
    Name = ODM.CodeList.Name
    DataType = ODM.CodeList.DataType
    SASFormatName = ODM.CodeList.SASFormatName
    Description = ODM.CodeList.Description
    CodeListItem = ODM.CodeList.CodeListItem
    EnumeratedItem = ODM.CodeList.EnumeratedItem
    ExternalCodeList = ODM.CodeList.ExternalCodeList
    Alias = ODM.CodeList.Alias


class MethodDef(ODM.MethodDef):
    OID = ODM.MethodDef.OID
    Name = ODM.MethodDef.Name
    Type = ODM.MethodDef.Type
    Description = ODM.MethodDef.Description
    FormalExpression = ODM.MethodDef.FormalExpression
    Alias = ODM.MethodDef.Alias
    DocumentRef = T.ODMListObject(element_class=DocumentRef, namespace="def")


class AnnotatedCRF(OE.ODMElement):
    namespace = "def"
    DocumentRef = T.ODMObject(required=True, element_class=DocumentRef, namespace="def")


class SupplementalDoc(OE.ODMElement):
    namespace = "def"
    DocumentRef = T.ODMListObject(required=True, element_class=DocumentRef, namespace="def")


class WhereClauseDef(OE.ODMElement):
    namespace = "def"
    OID = T.OID(required=True)
    CommentOID = T.OIDRef(namespace="def")
    RangeCheck = T.ODMListObject(required=True, element_class=RangeCheck)


class ValueListDef(OE.ODMElement):
    namespace = "def"
    OID = T.OID(required=True)
    ItemRef = T.ODMListObject(element_class=ItemRef, required=True)

    def __len__(self):
        return len(self.ItemRef)

    def __getitem__(self, position):
        return self.ItemRef[position]

    def __iter__(self):
        return iter(self.ItemRef)


class CommentDef(OE.ODMElement):
    namespace = "def"
    OID = T.OID(required=True)
    Description = T.ODMObject(element_class=Description)
    DocumentRef = T.ODMListObject(element_class=DocumentRef, namespace="def")


class MetaDataVersion(ODM.MetaDataVersion):
    OID = ODM.MetaDataVersion.OID
    Name = ODM.MetaDataVersion.Name
    Description = ODM.MetaDataVersion.Description
    DefineVersion = T.ValueSetString(required=True, namespace="def")
    StandardName = T.String(required=True, namespace="def")
    StandardVersion = T.String(required=True, namespace="def")
    AnnotatedCRF = T.ODMObject(element_class=AnnotatedCRF, namespace="def")
    SupplementalDoc = T.ODMObject(element_class=SupplementalDoc, namespace="def")
    ValueListDef = T.ODMListObject(element_class=ValueListDef, namespace="def")
    WhereClauseDef = T.ODMListObject(element_class=WhereClauseDef, namespace="def")
    ItemGroupDef = ODM.MetaDataVersion.ItemGroupDef
    ItemDef = ODM.MetaDataVersion.ItemDef
    CodeList = ODM.MetaDataVersion.CodeList
    MethodDef = ODM.MetaDataVersion.MethodDef
    CommentDef = T.ODMListObject(element_class=CommentDef, namespace="def")
    leaf = T.ODMListObject(element_class=leaf, namespace="def")


class Study(ODM.Study):
    OID = ODM.Study.OID
    GlobalVariables = ODM.Study.GlobalVariables
    MetaDataVersion = T.ODMObject(element_class=MetaDataVersion)


class ODM(ODM.ODM):
    FileType = ODM.ODM.FileType
    FileOID = ODM.ODM.FileOID
    CreationDateTime = ODM.ODM.CreationDateTime
    AsOfDateTime = ODM.ODM.AsOfDateTime
    ODMVersion = ODM.ODM.ODMVersion
    Originator = ODM.ODM.Originator
    SourceSystem = ODM.ODM.SourceSystem
    SourceSystemVersion = ODM.ODM.SourceSystemVersion
    schemaLocation = ODM.ODM.schemaLocation
    ID = ODM.ODM.ID
    Study = ODM.ODM.Study



import unittest
import model_extended as ODM


class TestExtendedAlias(unittest.TestCase):
    def test_valid_standard_attribute(self):
        ODM.Alias = ODM.Alias(Context="CDASH", Name="AEYN", Standard="CDASHIG")
        self.assertEqual(ODM.Alias.Standard, "CDASHIG")

    def test_invalid_standard_attribute(self):
        with self.assertRaises(TypeError):
            ODM.Alias = ODM.Alias(Context="CDASH", Name="AEYN", Standard="HL7 FHIR")


if __name__ == '__main__':
    unittest.main()



import unittest
import odmlib.odm_1_3_2.model as ODM
import datetime
import odmlib.odm_parser as ODM_PARSER
import xml.etree.ElementTree as ET
import odmlib.ns_registry as NS
import odmlib.odm_loader as OL
import odmlib.loader as LD

ODM_XML_FILE = "./data/simple_create.xml"
ODM_JSON_FILE = "./data/simple_create.json"


class MyTestCase(unittest.TestCase):
    def setUp(self) -> None:
        current_datetime = datetime.datetime.utcnow().replace(tzinfo=datetime.timezone.utc).isoformat()
        root = ODM.ODM(FileOID="ODM.DEMO.001", Granularity="Metadata", AsOfDateTime=current_datetime,
                       CreationDateTime=current_datetime, ODMVersion="1.3.2", FileType="Snapshot",
                       Originator="swhume", SourceSystem="odmlib", SourceSystemVersion="0.1")

        # create Study and add to ODM
        root.Study.append(ODM.Study(OID="ODM.GET.STARTED"))

        # create the global variables
        root.Study[0].GlobalVariables = ODM.GlobalVariables()
        root.Study[0].GlobalVariables.StudyName = ODM.StudyName(_content="Get Started with ODM XML")
        root.Study[0].GlobalVariables.StudyDescription = ODM.StudyDescription(
            _content="Demo to get started with odmlib")
        root.Study[0].GlobalVariables.ProtocolName = ODM.ProtocolName(_content="ODM XML Get Started")

        # create the MetaDataVersion
        root.Study[0].MetaDataVersion.append(ODM.MetaDataVersion(OID="MDV.DEMO-ODM-01", Name="Get Started MDV",
                                                                 Description="Get Started Demo"))
        # create Protocol
        p = ODM.Protocol()
        p.Description = ODM.Description()
        p.Description.TranslatedText.append(ODM.TranslatedText(_content="Get Started Protocol", lang="en"))
        p.StudyEventRef.append(ODM.StudyEventRef(StudyEventOID="BASELINE", OrderNumber=1, Mandatory="Yes"))
        root.Study[0].MetaDataVersion[0].Protocol = p

        # create a StudyEventDef
        sed = ODM.StudyEventDef(OID="BASELINE", Name="Baseline Visit", Repeating="No", Type="Scheduled")
        sed.FormRef.append(ODM.FormRef(FormOID="ODM.F.DM", Mandatory="Yes", OrderNumber=1))
        root.Study[0].MetaDataVersion[0].StudyEventDef.append(sed)

        # create a FormDef
        fd = ODM.FormDef(OID="ODM.F.DM", Name="Demographics", Repeating="No")
        fd.ItemGroupRef.append(ODM.ItemGroupRef(ItemGroupOID="ODM.IG.DM", Mandatory="Yes", OrderNumber=2))
        root.Study[0].MetaDataVersion[0].FormDef.append(fd)

        # create an ItemGroupDef
        igd = ODM.ItemGroupDef(OID="ODM.IG.DM", Name="Demographics", Repeating="No")
        igd.ItemRef.append(ODM.ItemRef(ItemOID="ODM.IT.DM.BRTHYR", Mandatory="Yes"))
        root.Study[0].MetaDataVersion[0].ItemGroupDef.append(igd)

        # create an ItemDef
        itd = ODM.ItemDef(OID="ODM.IT.DM.BRTHYR", Name="Birth Year", DataType="integer")
        itd.Description = ODM.Description()
        itd.Description.TranslatedText.append(ODM.TranslatedText(_content="Year of the subject's birth", lang="en"))
        itd.Question = ODM.Question()
        itd.Question.TranslatedText.append(ODM.TranslatedText(_content="Birth Year", lang="en"))
        itd.Alias.append(ODM.Alias(Context="CDASH", Name="BRTHYR"))
        itd.Alias.append(ODM.Alias(Context="SDTM", Name="BRTHDTC"))
        root.Study[0].MetaDataVersion[0].ItemDef.append(itd)

        # save the new ODM document to an ODM XML file
        root.write_xml(ODM_XML_FILE)

        # save the same ODM document to a JSON file
        root.write_json(ODM_JSON_FILE)


    def test_read_odm_xml(self):
        loader = LD.ODMLoader(OL.XMLODMLoader(model_package="odm_1_3_2", ns_uri="http://www.cdisc.org/ns/odm/v1.3"))
        loader.open_odm_document(ODM_XML_FILE)
        mdv = loader.MetaDataVersion()
        item_list = mdv.ItemDef
        item = item_list[0]
        self.assertEqual(item.OID, "ODM.IT.DM.BRTHYR")
        # tests the __len__ in ItemGroupDef
        self.assertEqual(len(item_list), 1)

    def test_read_odm_json(self):
        loader = LD.ODMLoader(OL.JSONODMLoader(model_package="odm_1_3_2"))
        loader.open_odm_document(ODM_JSON_FILE)
        mdv = loader.MetaDataVersion()
        igd_list = mdv.ItemGroupDef
        igd = igd_list[0]
        self.assertEqual(igd.OID, "ODM.IG.DM")
        # tests the __len__ in ItemGroupDef
        self.assertEqual(len(igd_list), 1)



import odmlib.document_loader as DL
import odmlib.odm_parser as P
import odmlib.ns_registry as NS
import json
import importlib
#TODO remove the import below
#import xml.etree.ElementTree as ET


class XMLDefineLoader(DL.DocumentLoader):
    def __init__(self, model_package="define_2_0", ns_uri="http://www.cdisc.org/ns/def/v2.0"):
        self.filename = None
        self.parser = None
        self.DEF = importlib.import_module(f"odmlib.{model_package}.model")
        self.ns_uri = ns_uri
        self.nsr = NS.NamespaceRegistry()

    def load_document(self, elem, *args):
        elem_name = elem.tag[elem.tag.find('}') + 1:]
        if elem.text and not elem.text.isspace():
            attrib = {**elem.attrib, **{"_content": elem.text}}
            odm_obj = eval("self.DEF." + elem_name + "(**" + str(attrib) + ")")
        else:
            odm_obj = eval("self.DEF." + elem_name + "(**" + str(elem.attrib) + ")")
        odm_obj_dict = eval("self.DEF." + elem_name + ".__dict__.items()")
        for k, v in odm_obj_dict:
            if type(v).__name__ == "ODMObject":
                namespace = self.nsr.get_ns_entry_dict(v.namespace)
                e = elem.find(v.namespace + ":" + k, namespace)
                if e is not None:
                    odm_child_obj = self.load_document(e)
                    exec("odm_obj." + k + " = odm_child_obj")
            elif type(v).__name__ == "ODMListObject":
                namespace = self.nsr.get_ns_entry_dict(v.namespace)
                for e in elem.findall(v.namespace + ":" + k, namespace):
                    odm_child_obj = self.load_document(e)
                    eval("odm_obj." + k + ".append(odm_child_obj)")
        return odm_obj

    def create_document(self, filename, namespace_registry=None):
        self.filename = filename
        self._set_registry(namespace_registry)
        self.parser = P.ODMParser(self.filename, self.nsr)
        root = self.parser.parse()
        return root

    def create_document_from_string(self, odm_string, namespace_registry=None):
        self._set_registry(namespace_registry)
        self.parser = P.ODMStringParser(odm_string, self.nsr)
        root = self.parser.parse()
        return root

    def _set_registry(self, namespace_registry):
        if namespace_registry:
            self.nsr = namespace_registry
        else:
            NS.NamespaceRegistry(prefix="odm", uri="http://www.cdisc.org/ns/odm/v1.3", is_default=True)
            self.nsr = NS.NamespaceRegistry(prefix="def", uri=self.ns_uri)

    def load_odm(self):
        root = self.parser.ODM()
        root_odmlib = self.load_document(root)
        return root_odmlib

    def load_metadataversion(self, idx=0):
        mdv = self.parser.MetaDataVersion()
        mdv_odmlib = self.load_document(mdv[idx])
        return mdv_odmlib

    def load_study(self, idx=0):
        study = self.parser.Study()
        # TODO will need to make Study a list and treat it like MDV above
        study_odmlib = self.load_document(study)
        return study_odmlib


class JSONDefineLoader(DL.DocumentLoader):
    def __init__(self, model_package="define_2_0"):
        self.filename = None
        self.odm_dict = {}
        self.DEF = importlib.import_module(f"odmlib.{model_package}.model")

    def load_document(self, odm_dict, key):
        attrib = {key: value for key, value in odm_dict.items() if not isinstance(value, (list, dict))}
        odm_obj = eval("self.DEF." + key + "(**" + str(attrib) + ")")
        odm_obj_items = eval("self.DEF." + key + ".__dict__.items()")
        for k, v in odm_obj_items:
            if type(v).__name__ == "ODMObject":
                if k in odm_dict:
                    odm_child_obj = self.load_document(odm_dict[k], k)
                    exec("odm_obj." + k + " = odm_child_obj")
            elif type(v).__name__ == "ODMListObject":
                if k in odm_dict:
                    for val in odm_dict[k]:
                        odm_child_obj = self.load_document(val, k)
                        eval("odm_obj." + k + ".append(odm_child_obj)")
        return odm_obj

    def create_document(self, filename):
        self.filename = filename
        with open(self.filename) as json_in:
            self.odm_dict = json.load(json_in)
        return self.odm_dict

    def create_document_from_string(self, odm_string):
        self.odm_dict = json.loads(odm_string)
        return self.odm_dict

    def load_odm(self):
        if not self.odm_dict:
            raise ValueError("create_document must be used to create the document before executing load_odm")
        odm_odmlib = self.load_document(self.odm_dict, "ODM")
        return odm_odmlib

    def load_metadataversion(self, idx=0):
        if not self.odm_dict:
            raise ValueError("create_document must be used to create the document before executing load_metadataversion")
        if "MetaDataVersion" in self.odm_dict:
            if isinstance(self.odm_dict["MetaDataVersion"], list):
                mdv_dict = self.odm_dict["MetaDataVersion"][idx]
            else:
                mdv_dict = self.odm_dict["MetaDataVersion"]
        elif "Study" in self.odm_dict and "MetaDataVersion" in self.odm_dict["Study"][0]:
            if isinstance(self.odm_dict["Study"][0]["MetaDataVersion"], list):
                mdv_dict = self.odm_dict["Study"][0]["MetaDataVersion"][idx]
            else:
                mdv_dict = self.odm_dict["Study"][0]["MetaDataVersion"]
        else:
            raise ValueError("MetaDataVersion not found in ODM dictionary")
        mdv_odmlib = self.load_document(mdv_dict, "MetaDataVersion")
        return mdv_odmlib

    def load_study(self, idx=0):
        if not self.odm_dict:
            raise ValueError("create_document must be used to create the document before executing load_study")
        elif "Study" in self.odm_dict:
            study_dict = self.odm_dict["Study"][idx]
        else:
            raise ValueError("Study not found in ODM dictionary")
        study_odmlib = self.load_document(study_dict, "Study")
        return study_odmlib



import unittest
import odmlib.odm_loader as OL
import odmlib.loader as LD
import os
import odmlib.oid_index as IDX


class TestOIDIndex(unittest.TestCase):
    def setUp(self) -> None:
        self.odm_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data', 'cdash-odm-test.xml')
        self.loader = LD.ODMLoader(OL.XMLODMLoader())
        self.loader.open_odm_document(self.odm_file)
        self.odm = self.loader.root()

    def test_oid_index_find_item(self):
        idx = self.odm.build_oid_index()
        found = idx.find_all("ODM.IT.DM.SEX")
        self.assertEqual(len(found), 2)
        self.assertEqual(found[0].ItemOID, "ODM.IT.DM.SEX")
        self.assertEqual(found[1].OID, "ODM.IT.DM.SEX")

    def test_oid_index_find_codelist(self):
        idx = self.odm.build_oid_index()
        found = idx.find_all("ODM.CL.NY_SUB_Y_N")
        self.assertEqual(len(found), 7)
        self.assertEqual(found[6].OID, "ODM.CL.NY_SUB_Y_N")
        self.assertEqual(found[1].CodeListOID, "ODM.CL.NY_SUB_Y_N")

    def test_without_oid_index_build(self):
        idx = IDX.OIDIndex()
        with self.assertRaises(ValueError):
            found = idx.find_all("ODM.CL.NY_SUB_Y_N")

    def test_not_found(self):
        idx = self.odm.build_oid_index()
        with self.assertRaises(ValueError):
            found = idx.find_all("ODM.CL.XXXX")



import odmlib.odm_element as OE
import odmlib.typed as T
import odmlib.ns_registry as NS


NS.NamespaceRegistry(prefix="odm", uri="http://www.cdisc.org/ns/odm/v1.3", is_default=True)
NS.NamespaceRegistry(prefix="xs", uri="http://www.w3.org/2001/XMLSchema-instance")
NS.NamespaceRegistry(prefix="xml", uri="http://www.w3.org/XML/1998/namespace")
NS.NamespaceRegistry(prefix="xlink", uri="http://www.w3.org/1999/xlink")


class TranslatedText(OE.ODMElement):
    lang = T.String(namespace="xml")
    _content = T.String(required=True)


class Alias(OE.ODMElement):
    Context = T.String(required=True)
    Name = T.String(required=True)
    Standard = T.ExtendedValidValues(required=True, valid_values=["CDASHIG", "SDTMIG"])


class StudyDescription(OE.ODMElement):
    _content = T.String(required=True)


class ProtocolName(OE.ODMElement):
    _content = T.String(required=True)


class StudyName(OE.ODMElement):
    _content = T.String(required=True)


class GlobalVariables(OE.ODMElement):
    StudyName = T.ODMObject(element_class=StudyName)
    StudyDescription = T.ODMObject(element_class=StudyDescription)
    ProtocolName = T.ODMObject(element_class=ProtocolName)


class Symbol(OE.ODMElement):
    TranslatedText = T.ODMListObject(required=True, element_class=TranslatedText)


class MeasurementUnit(OE.ODMElement):
    OID = T.OID(required=True)
    Name = T.Name(required=True)
    Symbol = T.ODMObject(required=True, element_class=Symbol)
    Alias = T.ODMListObject(element_class=Alias)


class BasicDefinitions(OE.ODMElement):
    MeasurementUnit = T.ODMListObject(element_class=MeasurementUnit)


class Include(OE.ODMElement):
    StudyOID = T.OIDRef(required=True)
    MetaDataVersionOID = T.OIDRef(required=True)


class Description(OE.ODMElement):
    TranslatedText = T.ODMListObject(required=True, element_class=TranslatedText)


class StudyEventRef(OE.ODMElement):
    StudyEventOID = T.OID(required=True)
    OrderNumber = T.Integer(required=False)
    Mandatory = T.ValueSetString(required=True)
    CollectionExceptionConditionOID = T.OIDRef()


class Protocol(OE.ODMElement):
    Description = T.ODMObject(element_class=Description)
    StudyEventRef = T.ODMListObject(element_class=StudyEventRef)
    Alias = T.ODMListObject(element_class=Alias)


class FormRef(OE.ODMElement):
    FormOID = T.OID(required=True)
    OrderNumber = T.Integer(required=False)
    Mandatory = T.ValueSetString(required=True)
    CollectionExceptionConditionOID = T.OIDRef()


class StudyEventDef(OE.ODMElement):
    OID = T.OID(required=True)
    Name = T.Name(required=True)
    Repeating = T.ValueSetString(required=True)
    Type = T.ValueSetString(required=True)
    Category = T.String(required=False)
    Description = T.ODMObject(element_class=Description)
    FormRef = T.ODMListObject(element_class=FormRef)
    Alias = T.ODMListObject(element_class=Alias)

    def __len__(self):
        """ returns the number of FormRefs in an StudyEventDef object as the length """
        return len(self.FormRef)

    def __getitem__(self, position):
        """ creates an iterator from an StudyEventDef object that returns the FormRef in position """
        return self.FormRef[position]

    def __iter__(self):
        return iter(self.FormRef)


class ItemGroupRef(OE.ODMElement):
    ItemGroupOID = T.OID(required=True)
    OrderNumber = T.Integer(required=False)
    Mandatory = T.ValueSetString(required=True)
    CollectionExceptionConditionOID = T.OIDRef()


class ArchiveLayout(OE.ODMElement):
    OID = T.OID(required=True)
    PdfFileName = T.FileName(required=True)
    PresentationOID = T.OIDRef(required=False)


class FormDef(OE.ODMElement):
    OID = T.OID(required=True)
    Name = T.Name(required=True)
    Repeating = T.ValueSetString(required=True)
    Description = T.ODMObject(element_class=Description)
    ItemGroupRef = T.ODMListObject(element_class=ItemGroupRef)
    ArchiveLayout = T.ODMListObject(element_class=ArchiveLayout)
    Alias = T.ODMListObject(element_class=Alias)

    def __len__(self):
        return len(self.ItemGroupRef)

    def __getitem__(self, position):
        return self.ItemGroupRef[position]

    def __iter__(self):
        return iter(self.ItemGroupRef)


class ItemRef(OE.ODMElement):
    ItemOID = T.String(required=True)
    OrderNumber = T.Integer(required=False)
    Mandatory = T.ValueSetString(required=True)
    KeySequence = T.Integer(required=False)
    MethodOID = T.String(required=False)
    Role = T.String()
    RoleCodeListOID = T.String()
    CollectionExceptionConditionOID = T.String()


class ItemGroupDef(OE.ODMElement):
    OID = T.OID(required=True)
    Name = T.Name(required=True)
    Repeating = T.ValueSetString(required=True)
    IsReferenceData = T.ValueSetString(required=False)
    SASDatasetName = T.SASName()
    Domain = T.String()
    Origin = T.String()
    Purpose = T.String()
    Comment = T.String()
    Description = T.ODMObject(element_class=Description)
    ItemRef = T.ODMListObject(element_class=ItemRef)
    Alias = T.ODMListObject(element_class=Alias)

    def __len__(self):
        return len(self.ItemRef)

    def __getitem__(self, position):
        return self.ItemRef[position]

    def __iter__(self):
        return iter(self.ItemRef)


class Question(OE.ODMElement):
    TranslatedText = T.ODMListObject(required=True, element_class=TranslatedText)


class ExternalQuestion(OE.ODMElement):
    Dictionary = T.String(required=False)
    Version = T.String(required=False)
    Code = T.String(required=False)


class MeasurementUnitRef(OE.ODMElement):
    MeasurementUnitOID = T.String(required=True)


class CheckValue(OE.ODMElement):
    _content = T.String(required=True)


class FormalExpression(OE.ODMElement):
    Context = T.String(required=True)
    _content = T.String(required=True)


class ErrorMessage(OE.ODMElement):
    TranslatedText = T.ODMListObject(required=True, element_class=TranslatedText)


class RangeCheck(OE.ODMElement):
    Comparator = T.ValueSetString(required=False)
    SoftHard = T.ValueSetString(required=True)
    CheckValue = T.ODMListObject(element_class=CheckValue)
    FormalExpression = T.ODMListObject(element_class=FormalExpression)
    MeasurementUnitRef = T.ODMObject(element_class=MeasurementUnitRef)
    ErrorMessage = T.ODMObject(element_class=ErrorMessage)


class CodeListRef(OE.ODMElement):
    CodeListOID = T.OIDRef(required=True)


class ItemDef(OE.ODMElement):
    OID = T.OID(required=True)
    Name = T.Name(required=True)
    DataType = T.ValueSetString(required=True)
    Length = T.PositiveInteger()
    SignificantDigits = T.NonNegativeInteger()
    SASFieldName = T.SASName()
    SDSVarName = T.SASName()
    Origin = T.String()
    Comment = T.String()
    Description = T.ODMObject(element_class=Description)
    Question = T.ODMObject(element_class=Question)
    ExternalQuestion = T.ODMObject(element_class=ExternalQuestion)
    MeasurementUnitRef = T.ODMListObject(element_class=MeasurementUnitRef)
    RangeCheck = T.ODMListObject(element_class=RangeCheck)
    CodeListRef = T.ODMObject(element_class=CodeListRef)
    Alias = T.ODMListObject(element_class=Alias)


class Decode(OE.ODMElement):
    TranslatedText = T.ODMListObject(required=True, element_class=TranslatedText)


class CodeListItem(OE.ODMElement):
    CodedValue = T.String(required=True)
    Rank = T.Float(required=False)
    OrderNumber = T.Integer(required=False)
    Decode = T.ODMObject(element_class=Decode)
    Alias = T.ODMListObject(element_class=Alias)


class EnumeratedItem(OE.ODMElement):
    CodedValue = T.String(required=True)
    Rank = T.Float(required=False)
    OrderNumber = T.Integer(required=False)
    Alias = T.ODMListObject(element_class=Alias)


class ExternalCodeList(OE.ODMElement):
    Dictionary = T.String(required=False)
    Version = T.String(required=False)
    ref = T.String(required=False)
    href = T.String(required=False)


class CodeList(OE.ODMElement):
    OID = T.OID(required=True)
    Name = T.Name(required=True)
    DataType = T.ValueSetString(required=True)
    SASFormatName = T.SASFormat()
    Description = T.ODMObject(element_class=Description)
    CodeListItem = T.ODMListObject(element_class=CodeListItem)
    EnumeratedItem = T.ODMListObject(element_class=EnumeratedItem)
    ExternalCodeList = T.ODMObject(element_class=ExternalCodeList)
    Alias = T.ODMListObject(element_class=Alias)


class Presentation(OE.ODMElement):
    OID = T.OID(required=True)
    lang = T.String(required=False)
    _content = T.String()


class ConditionDef(OE.ODMElement):
    OID = T.OID(required=True)
    Name = T.Name(required=True)
    Description = T.ODMObject(element_class=Description)
    FormalExpression = T.ODMListObject(element_class=FormalExpression)
    Alias = T.ODMListObject(element_class=Alias)


class MethodDef(OE.ODMElement):
    OID = T.OID(required=True)
    Name = T.Name(required=True)
    Type = T.ValueSetString(required=True)
    Description = T.ODMObject(required=True, element_class=Description)
    FormalExpression = T.ODMListObject(element_class=FormalExpression)
    Alias = T.ODMListObject(element_class=Alias)


class MetaDataVersion(OE.ODMElement):
    OID = T.OID(required=True)
    Name = T.Name(required=True)
    Description = T.String(required=False)
    Include = T.ODMObject(element_class=Include)
    Protocol = T.ODMObject(element_class=Protocol)
    StudyEventDef = T.ODMListObject(element_class=StudyEventDef)
    FormDef = T.ODMListObject(element_class=FormDef)
    ItemGroupDef = T.ODMListObject(element_class=ItemGroupDef)
    ItemDef = T.ODMListObject(element_class=ItemDef)
    CodeList = T.ODMListObject(element_class=CodeList)
    Presentation = T.ODMListObject(element_class=Presentation)
    ConditionDef = T.ODMListObject(element_class=ConditionDef)
    MethodDef = T.ODMListObject(element_class=MethodDef)


class LoginName(OE.ODMElement):
    _content = T.String(required=True)


class DisplayName(OE.ODMElement):
    _content = T.String(required=True)


class FullName(OE.ODMElement):
    _content = T.String(required=True)


class FirstName(OE.ODMElement):
    _content = T.String(required=True)


class LastName(OE.ODMElement):
    _content = T.String(required=True)


class Organization(OE.ODMElement):
    _content = T.String(required=True)


class StreetName(OE.ODMElement):
    _content = T.String(required=True)


class City(OE.ODMElement):
    _content = T.String(required=True)


class StateProv(OE.ODMElement):
    _content = T.String(required=True)


class Country(OE.ODMElement):
    _content = T.ValueSetString(required=True)


class PostalCode(OE.ODMElement):
    _content = T.String(required=True)


class OtherText(OE.ODMElement):
    _content = T.String(required=True)


class Address(OE.ODMElement):
    StreetName = T.ODMListObject(element_class=StreetName)
    City = T.ODMObject(element_class=City)
    StateProv = T.ODMObject(element_class=StateProv)
    Country = T.ODMObject(element_class=Country)
    PostalCode = T.ODMObject(element_class=PostalCode)
    OtherText = T.ODMObject(element_class=OtherText)


class Email(OE.ODMElement):
    _content = T.Email(required=True)


class Picture(OE.ODMElement):
    PictureFileName = T.FileName(required=True)
    ImageType = T.Name()


class Pager(OE.ODMElement):
    _content = T.String(required=True)


class Fax(OE.ODMElement):
    _content = T.String(required=True)


class Phone(OE.ODMElement):
    _content = T.String(required=True)


class LocationRef(OE.ODMElement):
    LocationOID = T.OIDRef(required=True)


class Certificate(OE.ODMElement):
    _content = T.String(required=True)


class User(OE.ODMElement):
    OID = T.OID(required=True)
    UserType = T.ValueSetString()
    LoginName = T.ODMObject(element_class=LoginName)
    DisplayName = T.ODMObject(element_class=DisplayName)
    FullName = T.ODMObject(element_class=FullName)
    FirstName = T.ODMObject(element_class=FirstName)
    LastName = T.ODMObject(element_class=LastName)
    Organization = T.ODMObject(element_class=Organization)
    Address = T.ODMListObject(element_class=Address)
    Email = T.ODMListObject(element_class=Email)
    Pager = T.ODMObject(element_class=Pager)
    Fax = T.ODMListObject(element_class=Fax)
    Phone = T.ODMListObject(element_class=Phone)
    LocationRef = T.ODMListObject(element_class=LocationRef)
    Certificate = T.ODMListObject(element_class=Certificate)


class MetaDataVersionRef(OE.ODMElement):
    StudyOID = T.OIDRef(required=True)
    MetaDataVersionOID = T.OIDRef(required=True)
    EffectiveDate = T.DateString(required=True)


class Location(OE.ODMElement):
    OID = T.OID(required=True)
    Name = T.Name(required=True)
    LocationType = T.ValueSetString()
    MetaDataVersionRef = T.ODMListObject(required=True, element_class=MetaDataVersionRef)


class Meaning(OE.ODMElement):
    _content = T.String(required=True)


class LegalReason(OE.ODMElement):
    _content = T.String(required=True)


class SignatureDef(OE.ODMElement):
    OID = T.OID(required=True)
    Methodology = T.ValueSetString()
    Meaning = T.ODMObject(required=True, element_class=Meaning)
    LegalReason = T.ODMObject(required=True, element_class=LegalReason)


class AdminData(OE.ODMElement):
    StudyOID = T.OIDRef()
    User = T.ODMListObject(element_class=User)
    Location = T.ODMListObject(element_class=Location)
    SignatureDef = T.ODMListObject(element_class=SignatureDef)


class Study(OE.ODMElement):
    OID = T.String(required=True)
    GlobalVariables = T.ODMObject(required=True, element_class=GlobalVariables)
    BasicDefinitions = T.ODMObject(element_class=BasicDefinitions)
    MetaDataVersion = T.ODMListObject(required=False, element_class=MetaDataVersion)


class UserRef(OE.ODMElement):
    UserOID = T.OIDRef(required=True)


class DateTimeStamp(OE.ODMElement):
    _content = T.DateTimeString(required=True)


class ReasonForChange(OE.ODMElement):
    _content = T.String(required=True)


class SourceID(OE.ODMElement):
    _content = T.String(required=True)


class AuditRecord(OE.ODMElement):
    EditPoint = T.ValueSetString(required=False)
    UsedImputationMethod = T.ValueSetString(required=False)
    ID = T.ID(required=False)
    UserRef = T.ODMObject(required=True, element_class=UserRef)
    LocationRef = T.ODMObject(required=True, element_class=LocationRef)
    DateTimeStamp = T.ODMObject(required=True, element_class=DateTimeStamp)
    ReasonForChange = T.ODMObject(required=False, element_class=ReasonForChange)
    SourceID = T.ODMObject(required=False, element_class=SourceID)


class SignatureRef(OE.ODMElement):
    SignatureOID = T.OIDRef(required=True)


class Signature(OE.ODMElement):
    ID = T.ID(required=True)
    UserRef = T.ODMObject(required=True, element_class=UserRef)
    LocationRef = T.ODMObject(required=True, element_class=LocationRef)
    SignatureRef = T.ODMObject(required=True, element_class=SignatureRef)
    DateTimeStamp = T.ODMObject(required=True, element_class=DateTimeStamp)


class InvestigatorRef(OE.ODMElement):
    UserOID = T.OIDRef(required=True)


class SiteRef(OE.ODMElement):
    LocationOID = T.OIDRef(required=True)


class FlagValue(OE.ODMElement):
    CodeListOID = T.OIDRef(required=True)
    _content = T.String(required=True)


class FlagType(OE.ODMElement):
    CodeListOID = T.OIDRef(required=True)
    _content = T.String(required=True)


class Flag(OE.ODMElement):
    FlagValue = T.ODMObject(required=True, element_class=FlagValue)
    FlagType = T.ODMObject(element_class=FlagType)


class Comment(OE.ODMElement):
    SponsorOrSite = T.ValueSetString()
    _content = T.String(required=True)


class Annotation(OE.ODMElement):
    SeqNum = T.Integer(required=True)
    TransactionType = T.ValueSetString(required=False)
    ID = T.ID(required=False)
    Comment = T.ODMObject(required=False, element_class=Comment)
    Flag = T.ODMListObject(element_class=Flag)


class ItemData(OE.ODMElement):
    ItemOID = T.OIDRef(required=True)
    TransactionType = T.ValueSetString(required=False)
    Value = T.String(required=False)
    IsNull = T.ValueSetString(required=False)
    AuditRecord = T.ODMObject(required=False, element_class=AuditRecord)
    Signature = T.ODMObject(required=False, element_class=Signature)
    MeasurementUnitRef = T.ODMObject(required=False, element_class=MeasurementUnitRef)
    Annotation = T.ODMListObject(required=False, element_class=Annotation)


class ItemGroupData(OE.ODMElement):
    ItemGroupOID = T.OIDRef(required=True)
    ItemGroupRepeatKey = T.String(required=False)
    TransactionType = T.ValueSetString(required=False)
    AuditRecord = T.ODMObject(required=False, element_class=AuditRecord)
    Signature = T.ODMObject(required=False, element_class=Signature)
    ArchiveLayout = T.ODMObject(required=False, element_class=ArchiveLayout)
    Annotation = T.ODMListObject(required=False, element_class=Annotation)
    ItemData = T.ODMListObject(required=False, element_class=ItemData)


class FormData(OE.ODMElement):
    FormOID = T.OIDRef(required=True)
    FormRepeatKey = T.String(required=False)
    TransactionType = T.ValueSetString(required=False)
    AuditRecord = T.ODMObject(required=False, element_class=AuditRecord)
    Signature = T.ODMObject(required=False, element_class=Signature)
    ArchiveLayout = T.ODMObject(required=False, element_class=ArchiveLayout)
    Annotation = T.ODMListObject(required=False, element_class=Annotation)
    ItemGroupData = T.ODMListObject(required=False, element_class=ItemGroupData)


class StudyEventData(OE.ODMElement):
    StudyEventOID = T.OIDRef(required=True)
    StudyEventRepeatKey = T.String(required=False)
    TransactionType = T.ValueSetString(required=False)
    AuditRecord = T.ODMObject(required=False, element_class=AuditRecord)
    Signature = T.ODMObject(required=False, element_class=Signature)
    Annotation = T.ODMListObject(required=False, element_class=Annotation)
    FormData = T.ODMListObject(required=False, element_class=FormData)


class SubjectData(OE.ODMElement):
    SubjectKey = T.String(required=True)
    TransactionType = T.ValueSetString(required=False)
    AuditRecord = T.ODMObject(required=False, element_class=AuditRecord)
    Signature = T.ODMObject(required=False, element_class=Signature)
    InvestigatorRef = T.ODMObject(required=False, element_class=InvestigatorRef)
    SiteRef = T.ODMObject(required=False, element_class=SiteRef)
    Annotation = T.ODMListObject(required=False, element_class=Annotation)
    StudyEventData = T.ODMListObject(required=False, element_class=StudyEventData)


class AuditRecords(OE.ODMElement):
    AuditRecord = T.ODMListObject(required=False, element_class=AuditRecord)


class Signatures(OE.ODMElement):
    Signature = T.ODMListObject(required=False, element_class=Signature)


class Annotations(OE.ODMElement):
    Annotation = T.ODMListObject(required=False, element_class=Annotation)


class ClinicalData(OE.ODMElement):
    StudyOID = T.OIDRef(required=True)
    MetaDataVersionOID = T.OIDRef(required=True)
    SubjectData = T.ODMListObject(element_class=SubjectData)
    AuditRecords = T.ODMListObject(element_class=AuditRecords)
    Signatures = T.ODMListObject(element_class=Signatures)
    Annotations = T.ODMListObject(element_class=Annotations)


class ReferenceData(OE.ODMElement):
    StudyOID = T.OIDRef(required=True)
    MetaDataVersionOID = T.OIDRef(required=True)
    ItemGroupData = T.ODMListObject(element_class=ItemGroupData)
    AuditRecords = T.ODMListObject(element_class=AuditRecords)
    Signatures = T.ODMListObject(element_class=Signatures)
    Annotations = T.ODMListObject(element_class=Annotations)


class KeySet(OE.ODMElement):
    StudyOID = T.OIDRef(required=True)
    SubjectKey = T.String()
    StudyEventOID = T.OIDRef()
    StudyEventRepeatKey = T.String()
    FormOID = T.OIDRef()
    FormRepeatKey = T.String()
    ItemGroupOID = T.OIDRef()
    ItemGroupRepeatKey = T.String()
    ItemOID = T.OIDRef()


class Association(OE.ODMElement):
    StudyOID = T.OIDRef(required=True)
    MetaDataVersionOID = T.OIDRef(required=True)
    KeySet = T.ODMListObject(required=True, element_class=KeySet)
    Annotation = T.ODMObject(required=True, element_class=Annotation)


class ODM(OE.ODMElement):
    Description = T.String(required=False)
    FileType = T.ValueSetString(required=True)
    Granularity = T.ValueSetString(required=False)
    Archival = T.ValueSetString(required=False)
    FileOID = T.OID(required=True)
    CreationDateTime = T.DateTimeString(required=True)
    PriorFileOID = T.OIDRef(required=False)
    AsOfDateTime = T.DateTimeString(required=False)
    ODMVersion = T.ValueSetString(required=False)
    Originator = T.String(required=False)
    SourceSystem = T.String(required=False)
    SourceSystemVersion = T.String(required=False)
    schemaLocation = T.String(required=False, namespace="xs")
    ID = T.ID()
    Study = T.ODMListObject(element_class=Study)
    AdminData = T.ODMListObject(element_class=AdminData)
    ReferenceData = T.ODMListObject(element_class=ReferenceData)
    ClinicalData = T.ODMListObject(element_class=ClinicalData)
    Association = T.ODMListObject(element_class=Association)
